#include <time.h>
#include <sys/time.h>
#include <unistd.h>

#include "ship.h"
#include "objglobals.h"
#include "netvariables.h"
#include "netdata.h"
#include "network.h"
#include "thread.h"


struct timeval LastTime;
struct timeval UpdateTime;


void RunFrame()
{
  int i, j, PlayersLeft, RunTime;
  struct timezone tz;
  struct timeval now;

  // Play the boom sounds
  if (--NextBoom == 0) {
    if (BoomPhase) {
      playsound(Boom1);
      BoomPhase = 0;
    } else {
      playsound(Boom2);
      BoomPhase = 1;
    }
    NextBoom = BoomDelay;
  }

  // Display object and ship sprites
  for (i=0; i<MAX_OBJECTS; i++) {
    if (Sprites[i]!=NULL) {
      Sprites[i]->ShowSprite(Ships[myShip]->xpos, Ships[myShip]->ypos);
    }
  }

  for (i=0; i<NumPlayers; i++) {
    Ships[i]->ShowSprite(Ships[myShip]->xpos, Ships[myShip]->ypos);
  }

  gettimeofday(&now, &tz);

  // Send out update every second
  if (now.tv_sec!=UpdateTime.tv_sec) {
    SendUpdate();
    memcpy(&UpdateTime, &now, sizeof(struct timeval));
  }
  pthread_mutex_unlock(ClientLock);

  /*
    We need to sleep to accomplish the following:

    Make sure that each frame update takes the same amount of time
    Handle the sound correctly
    Allow other threads to update the data
    Not be to much of a CPU hog
  */
  RunTime=(now.tv_sec-LastTime.tv_sec)*1000000+(now.tv_usec-LastTime.tv_usec);
  LastTime.tv_sec=now.tv_sec;
  LastTime.tv_usec=now.tv_usec;

  // The Cygwin usleep and gettimeofday functions appear to be broken,
  // so this is a workaround
#ifndef CYGWIN
  if (RunTime<FRAME_DELAY*1000 && RunTime>0) {
    usleep(FRAME_DELAY*1000-RunTime);
  }
#else
  usleep(FRAME_DELAY*1000);
#endif

  gettimeofday(&LastTime, &tz);

  pthread_mutex_lock(ClientLock);
}
