#include "globals.h"
#include "radar.h"
#include "input.h"
#include "fileutils.h"


int ScreenWidth, ScreenHeight;
char ServerIP[16];
int ServerPort;
int svrsock;
char PlayerName[MAX_PLAYERNAME_LENGTH];
char ConfigFileName[80];


void ReadConfig();
void SetDefaults();
int GetWord(FILE *fileptr, char *Word, char EndOfWord, int MaxSize);
void AssignValue(char *VariableName, char *Value);
int IgnoreCommentsAndSpaces(FILE *fileptr, char *c);
void TerminateVariable(char *variable);
int GoToEndOfLine(FILE *fileptr);


// Function to read the configuration file
void ReadConfig()
{
  FILE *ConfigFile;

  SetDefaults();

  if (!(ConfigFile=fopen(ConfigFileName, "r"))) {
    return;
  }

  ReadVariables(ConfigFile, &AssignValue);

  fclose(ConfigFile);
}


void SetDefaults()
{
  ScreenWidth=640;
  ScreenHeight=480;
  strcpy(ServerIP, "127.0.0.1");
  ServerPort=12345;
  strcpy(PlayerName, "UnNamedPlayer");
  RadarWidth=128;
  RadarHeight=96;
}


void AssignValue(char *VariableName, char *Value)
{
  int temp;

  if (!strcmp(VariableName, "SCREEN_WIDTH")) {
    temp=atoi(Value);
    if (temp >= 320 && temp <= 1600) {
      ScreenWidth=temp;
    }
  } else if (!strcmp(VariableName, "SCREEN_HEIGHT")) {
    temp=atoi(Value);
    if (temp >= 240 && temp <= 1200) {
      ScreenHeight=temp;
    }
  } else if (!strcmp(VariableName, "RADAR_WIDTH")) {
    temp=atoi(Value);
    if (temp >= 32 && temp <= 160) {
      RadarWidth=temp;
    }
  } else if (!strcmp(VariableName, "RADAR_HEIGHT")) {
    temp=atoi(Value);
    if (temp >= 24 && temp <= 120) {
      RadarHeight=temp;
    }
  } else if (!strcmp(VariableName, "PLAYER_NAME")) {
    if (strlen(Value) < MAX_PLAYERNAME_LENGTH && strlen(Value)) {
      strcpy(PlayerName, Value);
    }
  } else if (!strcmp(VariableName, "SERVER_IP")) {
    strcpy(ServerIP, Value);
  } else if (!strcmp(VariableName, "SERVER_PORT")) {
    ServerPort=atoi(Value);
  } else if (!strcmp(VariableName, "THRUST_KEY")) {
    controls.Thrust=*Value;
  } else if (!strcmp(VariableName, "RIGHT_KEY")) {
    controls.Right=*Value;
  } else if (!strcmp(VariableName, "LEFT_KEY")) {
    controls.Left=*Value;
  } else if (!strcmp(VariableName, "FIRE_KEY")) {
    controls.Fire=*Value;
  } else if (!strcmp(VariableName, "SHIELD_KEY")) {
    controls.Shield=*Value;
  } else if (!strcmp(VariableName, "QUIT_KEY")) {
    controls.Quit=*Value;
  } else if (!strcmp(VariableName, "SCORE_KEY")) {
    controls.Score=*Value;
  }
}


// Save Configuration to a file
void SaveConfig()
{
  FILE *ConfigFile;

  if (!(ConfigFile=fopen(ConfigFileName, "w"))) {
    return;
  }

  fprintf(ConfigFile, "SERVER_IP=%s\n", ServerIP);
  fprintf(ConfigFile, "SERVER_PORT=%d\n", ServerPort);
  fprintf(ConfigFile, "SCREEN_WIDTH=%d\n", ScreenWidth);
  fprintf(ConfigFile, "SCREEN_HEIGHT=%d\n", ScreenHeight);
  fprintf(ConfigFile, "RADAR_WIDTH=%d\n", RadarWidth);
  fprintf(ConfigFile, "RADAR_HEIGHT=%d\n", RadarHeight);
  fprintf(ConfigFile, "PLAYER_NAME=%s\n", PlayerName);
  fprintf(ConfigFile, "LEFT_KEY=%c\n", controls.Left);
  fprintf(ConfigFile, "RIGHT_KEY=%c\n", controls.Right);
  fprintf(ConfigFile, "FIRE_KEY=%c\n", controls.Fire);
  fprintf(ConfigFile, "SHIELD_KEY=%c\n", controls.Shield);
  fprintf(ConfigFile, "THRUST_KEY=%c\n", controls.Thrust);
  fprintf(ConfigFile, "SCORE_KEY=%c\n", controls.Score);
  fprintf(ConfigFile, "QUIT_KEY=%c\n", controls.Quit);

  fclose(ConfigFile);
}
