#include <time.h>
#include <sys/time.h>
#include <unistd.h>

#include "ship.h"
#include "objglobals.h"
#include "sizes.h"
#include "network.h"
#include "netvariables.h"
#include "readserverthread.h"
#include "status.h"
#include "blit.h"
#include "timevar.h"
#include "thread.h"


int BoomDelay;
int NextBoom;
int BoomPhase;
pthread_mutex_t *ClientLock;
pthread_t readthread;
int GameStop;


void NewGame();
void RunGame();
void DoHouseKeeping();
void InitGame();


void NewGame()
{
  struct timezone tz;

  // Connect to game server
  ConnectServer();

  InitGame();

  gettimeofday(&LastTime, &tz);
  gettimeofday(&UpdateTime, &tz);
}


void RunGame()
{
  ClearScreen();

  RunFrame();

  DrawStars();
  DrawBlits();
  DrawStatus();

  UpdateScreen();
}


void InitGame()
{
  int i, rc;
  pthread_attr_t attr;

  for (i=0; i<NumPlayers; i++) {
    Ships[i]=new Ship(i);
  }

  GameStop=0;

  if (!ClientLock) {
    ClientLock=(pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    pthread_mutex_init(ClientLock, NULL);
  }
  pthread_mutex_lock(ClientLock);

  // Create a thread to listen for data from the server and update game data
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, !PTHREAD_CREATE_DETACHED);
  if (rc = pthread_create(&readthread, &attr, ReadServerThread,
			  (void *)svrsock)) {
    printf("ERROR, return code from pthread_create() is %d\n", rc);
    pthread_exit(NULL);
  }
  pthread_attr_destroy(&attr);

  BoomDelay = 800/FRAME_DELAY;
  NextBoom = BoomDelay;
  BoomPhase = 0;
}
