#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>


#include "ship.h"
#include "sizes.h"
#include "status.h"
#include "score.h"
#include "thread.h"
#include "clientconfig.h"
#include "serverconfig.h"


Object *Sprites[MAX_OBJECTS];
Ship *Ships[MAX_PLAYERS];
int NumPlayers;
int myShip;
int GameOver;
MPoint ThrustOrigins[SHIP_FRAMES];
Sprite *BigRockSprite, *MediumRockSprite, *SmallRockSprite,
  *DamagedShipSprite, *ShipExplosionSprite, *MultiplierSprite[4], *ShipSprite,
  *ExplosionSprite, *NovaSprite, *EnemyShipSprite, *EnemyShip2Sprite,
  *SteelRoidSprite, *PrizeSprite, *BonusSprite, *PointSprite,
  *VortexSprite, *HomingMineSprite, *ShieldSprite, *Thrust1Sprite,
  *Thrust2Sprite, *Shrapnel1Sprite, *Shrapnel2Sprite, *ShotSprite,
  *TripleIconSprite, *LuckIconSprite, *AutoFireIconSprite, *LongFireIconSprite,
  *SpeedupIconSprite;
KeyValuesStruct KeyValues;


void LoadSprites();
Sprite *LoadSprite(char *dirname, int frames);
void LoadBMP(char *filename, unsigned long *width,
	     unsigned long *height, unsigned char *pixels);
void InitKeyValues();
int copyfile(char *ConfDirName, char *FileName);


// Initialize the position of the thrust relative to the ship
void InitThrust()
{
  ThrustOrigins[0].h = 8;
  ThrustOrigins[0].v = -6;

  ThrustOrigins[1].h = 6;
  ThrustOrigins[1].v = -6;

  ThrustOrigins[2].h = 4;
  ThrustOrigins[2].v = -5;

  ThrustOrigins[3].h = 1;
  ThrustOrigins[3].v = -4;

  ThrustOrigins[4].h = 0;
  ThrustOrigins[4].v = -3;

  ThrustOrigins[5].h = -1;
  ThrustOrigins[5].v = -3;

  ThrustOrigins[6].h = -3;
  ThrustOrigins[6].v = 0;

  ThrustOrigins[7].h = -5;
  ThrustOrigins[7].v = 1;

  ThrustOrigins[8].h = -6;
  ThrustOrigins[8].v = 3;

  ThrustOrigins[9].h = -9;
  ThrustOrigins[9].v = 5;

  ThrustOrigins[10].h = -10;
  ThrustOrigins[10].v = 6;

  ThrustOrigins[11].h = -11;
  ThrustOrigins[11].v = 9;

  ThrustOrigins[12].h = -9;
  ThrustOrigins[12].v = 9;

  ThrustOrigins[13].h = -9;
  ThrustOrigins[13].v = 12;

  ThrustOrigins[14].h = -7;
  ThrustOrigins[14].v = 14;

  ThrustOrigins[15].h = -6;
  ThrustOrigins[15].v = 16;

  ThrustOrigins[16].h = -9;
  ThrustOrigins[16].v = 15;

  ThrustOrigins[17].h = -3;
  ThrustOrigins[17].v = 19;

  ThrustOrigins[18].h = -1;
  ThrustOrigins[18].v = 18;

  ThrustOrigins[19].h = 0;
  ThrustOrigins[19].v = 20;

  ThrustOrigins[20].h = 4;
  ThrustOrigins[20].v = 22;

  ThrustOrigins[21].h = 5;
  ThrustOrigins[21].v = 24;

  ThrustOrigins[22].h = 5;
  ThrustOrigins[22].v = 22;

  ThrustOrigins[23].h = 8;
  ThrustOrigins[23].v = 23;

  ThrustOrigins[24].h = 9;
  ThrustOrigins[24].v = 23;

  ThrustOrigins[25].h = 12;
  ThrustOrigins[25].v = 22;

  ThrustOrigins[26].h = 13;
  ThrustOrigins[26].v = 22;

  ThrustOrigins[27].h = 15;
  ThrustOrigins[27].v = 23;

  ThrustOrigins[28].h = 17;
  ThrustOrigins[28].v = 22;

  ThrustOrigins[29].h = 18;
  ThrustOrigins[29].v = 20;

  ThrustOrigins[30].h = 20;
  ThrustOrigins[30].v = 18;

  ThrustOrigins[31].h = 19;
  ThrustOrigins[31].v = 17;

  ThrustOrigins[32].h = 21;
  ThrustOrigins[32].v = 16;

  ThrustOrigins[33].h = 22;
  ThrustOrigins[33].v = 14;

  ThrustOrigins[34].h = 24;
  ThrustOrigins[34].v = 15;

  ThrustOrigins[35].h = 25;
  ThrustOrigins[35].v = 11;

  ThrustOrigins[36].h = 26;
  ThrustOrigins[36].v = 9;

  ThrustOrigins[37].h = 25;
  ThrustOrigins[37].v = 9;

  ThrustOrigins[38].h = 24;
  ThrustOrigins[38].v = 6;

  ThrustOrigins[39].h = 23;
  ThrustOrigins[39].v = 5;

  ThrustOrigins[40].h = 23;
  ThrustOrigins[40].v = 4;

  ThrustOrigins[41].h = 20;
  ThrustOrigins[41].v = 2;

  ThrustOrigins[42].h = 20;
  ThrustOrigins[42].v = 0;

  ThrustOrigins[43].h = 18;
  ThrustOrigins[43].v = -2;

  ThrustOrigins[44].h = 15;
  ThrustOrigins[44].v = -2;

  ThrustOrigins[45].h = 15;
  ThrustOrigins[45].v = -4;

  ThrustOrigins[46].h = 12;
  ThrustOrigins[46].v = -5;

  ThrustOrigins[47].h = 9;
  ThrustOrigins[47].v = -6;

}


void DoInitializations()
{
  int i;

  for (i=0; i<MAX_OBJECTS; i++) {
    Sprites[i]=NULL;
  }

  InitScreen();

  playsound(PrizeAppears);

  // Load the sprites into memory
  LoadSprites();

  InitColors();
  InitThrust();
  InitStars();
  InitStatus();
  InitHighScores();
  ReadHighScores();

  NumBlits=0;
  Lives=0;
  GameOver=0;
  ShowScore=0;
  ServerStarted=0;
  ClientLock=NULL;
  ServerLock=NULL;
}


void LoadSprites()
{
  int i;
  char filename[80];

  Thrust1Sprite = LoadSprite("bmps/thrust1", SHIP_FRAMES);
  Thrust2Sprite = LoadSprite("bmps/thrust2", SHIP_FRAMES);
  ShipSprite = LoadSprite("bmps/ship", SHIP_FRAMES);
  ShotSprite = LoadSprite("bmps/shot", 1);
  BigRockSprite = LoadSprite("bmps/bigrock", 60);
  MediumRockSprite = LoadSprite("bmps/mediumrock", 40);
  SmallRockSprite = LoadSprite("bmps/smallrock", 20);
  ExplosionSprite = LoadSprite("bmps/explosion", 12);
  MultiplierSprite[0] = LoadSprite("bmps/2x", 1);
  MultiplierSprite[1] = LoadSprite("bmps/3x", 1);
  MultiplierSprite[2] = LoadSprite("bmps/4x", 1);
  MultiplierSprite[3] = LoadSprite("bmps/5x", 1);
  SteelRoidSprite = LoadSprite("bmps/steelroid", 40);
  PrizeSprite = LoadSprite("bmps/prize", 30);
  BonusSprite = LoadSprite("bmps/comet", 10);
  PointSprite = LoadSprite("bmps/bonus", 6);
  VortexSprite = LoadSprite("bmps/vortex", 10);
  HomingMineSprite = LoadSprite("bmps/homing", 40);
  ShieldSprite = LoadSprite("bmps/shield", 2);
  NovaSprite = LoadSprite("bmps/nova", 18);
  ShipExplosionSprite = LoadSprite("bmps/shipexplosion", 21);
  Shrapnel1Sprite = LoadSprite("bmps/shrapnel1", 50);
  Shrapnel2Sprite = LoadSprite("bmps/shrapnel2", 42);
  DamagedShipSprite = LoadSprite("bmps/damagedship", 10);
  EnemyShipSprite = LoadSprite("bmps/bigenemyship", 40);
  EnemyShip2Sprite = LoadSprite("bmps/smallenemyship", 40);
  TripleIconSprite = LoadSprite("bmps/tripleicon", 1);
  LuckIconSprite = LoadSprite("bmps/luckicon", 1);
  AutoFireIconSprite = LoadSprite("bmps/autofireicon", 1);
  LongFireIconSprite = LoadSprite("bmps/longfireicon", 1);
  SpeedupIconSprite = LoadSprite("bmps/speedupicon", 1);
}


Sprite *LoadSprite(char *dirname, int frames)
{
  char filename[80];
  Sprite *newSprite;
  int i;

  newSprite = new Sprite;
  newSprite->Frames=frames;

  for (i=0; i<newSprite->Frames; i++) {
#ifdef WINDOWS
    sprintf(filename, "%s/%s/%02d.bmp", DATADIR, dirname, i);
#else
    sprintf(filename, "%s/atr/%s/%02d.bmp", DATADIR, dirname, i);
#endif
    newSprite->pixels[i]=(unsigned char *)malloc(MAX_SPRITE_SIZE*MAX_SPRITE_SIZE*3);
    LoadBMP(filename, &newSprite->Width,
	    &newSprite->Height, newSprite->pixels[i]);
  }
  return(newSprite);
}


/*
  This function reads the bmp file 'filename' into the 'pixels' variable.
  It can only handle an uncompressed, 24 bit color bmp file.
  For information on the bmp file format, read the file BMP.txt in the docs
  directory.
*/
void LoadBMP(char *filename, unsigned long *width,
	     unsigned long *height, unsigned char *pixels)
{
  int fd, i, j, k, offset;
  unsigned char c;
  char *ptr;
  long endiantest=1;
  int ChangeEndian;

  // Check if CPU is the right endian for bmp files
  ptr=(char *)(&endiantest);
  if (*ptr==1) {
    ChangeEndian=FALSE;
  } else {
    ChangeEndian=TRUE;
  }

  fd=open(filename, O_RDONLY);

  for (i=0; i<18; i++) {
    read(fd, &c, 1);
  }

  read(fd, width, 4);
  read(fd, height, 4);

  if (ChangeEndian) {
    // width and height are stored in the wrong endian format, so they have to
    // be changed.
    ptr=(char *)width;
    c=*ptr;
    *ptr=*(ptr+3);
    *(ptr+3)=c;
    c=*(ptr+1);
    *(ptr+1)=*(ptr+2);
    *(ptr+2)=c;
    ptr=(char *)height;
    c=*ptr;
    *ptr=*(ptr+3);
    *(ptr+3)=c;
    c=*(ptr+1);
    *(ptr+1)=*(ptr+2);
    *(ptr+2)=c;
  }

  for (i=0; i<28; i++) {
    read(fd, &c, 1);
  }

  offset=(4-*width%4)%4;
  for (i=0; i<*height; i++) {
    for (j=0; j<*width; j++) {
      read(fd, pixels, 3);
      // Switch red and blue pixels to convert from rgb to bgr
      c=*pixels;
      *pixels=*(pixels+2);
      *(pixels+2)=c;
      pixels+=3;
    }
    for (k=0; k<offset; k++) {
      read(fd, &c, 1);
    }
  }
  close(fd);
}


void InitKeyValues()
{
  KeyValues.ThrustKey=0;
  KeyValues.RightKey=0;
  KeyValues.LeftKey=0;
  KeyValues.FireKey=0;
  KeyValues.ShieldKey=0;
  KeyValues.AbortKey=0;
}


void SetupConfiguration()
{
  DIR *dir;
  char ConfDirName[80];

#ifdef WINDOWS
  sprintf(ConfDirName, "%s/conf", DATADIR);
#else
  sprintf(ConfDirName, "%s/.atr", getenv("HOME"));

  if (!(dir=opendir(ConfDirName))) {
    if (mkdir(ConfDirName, 0755)) {
      printf("Error opening atr directory\n");
      exit(1);
    }
    if (!copyfile(ConfDirName, "scores.dat")) {
      printf("Error creating scores.dat file\n");
      exit(1);
    }
    if (!copyfile(ConfDirName, "ATRConfig")) {
      printf("Error creating ATRConfig file\n");
      exit(1);
    }
    if (!copyfile(ConfDirName, "ATRServerConfig")) {
      printf("Error creating ATRServerConfig file\n");
      exit(1);
    }
    if (!copyfile(ConfDirName, "ATRServerSingleConfig")) {
      printf("Error creating ATRServerSingleConfig file\n");
      exit(1);
    }
  } else {
    closedir(dir);
  }
#endif

  sprintf(ScoreFileName, "%s/scores.dat", ConfDirName);
  sprintf(ConfigFileName, "%s/ATRConfig", ConfDirName);
  sprintf(ServerConfigFileName, "%s/ATRServerConfig", ConfDirName);
  sprintf(ServerSingleConfigFileName, "%s/ATRServerSingleConfig", ConfDirName);
}


int copyfile(char *ConfDirName, char *FileName)
{
  FILE *oldfd, *newfd;
  char oldFileName[80], newFileName[80];
  int c;

  sprintf(oldFileName, "%s/atr/conf/%s", DATADIR, FileName);
  sprintf(newFileName, "%s/%s", ConfDirName, FileName);

  if (!(oldfd=fopen(oldFileName, "r"))) {
    return(0);
  }
  if (!(newfd=fopen(newFileName, "w"))) {
    fclose(oldfd);
    return(0);
  }
  while ((c=fgetc(oldfd))!=EOF) {
    fputc(c, newfd);
  }
  fclose(oldfd);
  fclose(newfd);

  return(1);
}
