#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "defines.h"
#include "thread.h"
#include "serverdata.h"
#include "servergame.h"
#include "netdata.h"
#include "netconfig.h"
#include "servership.h"
#include "objdata.h"
#include "sounddefs.h"
#include "serversound.h"


ServerShip *ServerShips[MAX_PLAYERS];


void *MainPlayerThread(void *threadid);
void SetupShipSendData(int playernum, ShipDataStruct *ShipData);
void SetupObjSendData(int objnum, ObjDataStruct *ObjData);
void SendPlayerData(int threadnum);
void SetupStatusSendData(char *GameStatus, int playernum);
void TransferData(unsigned char *SendData, int *DataSize, int threadnum);


void *MainPlayerThread(void *threadid)
{
  int n, rc;
  int threadnum;
  pthread_t readthread;

  threadnum=(int)threadid;

  while (!GameStart) {
    usleep(NetSleepTime);
  }

  pthread_mutex_lock(ServerLock);
  InitPlayerData(threadnum);
  Connections[threadnum].Connected=1;
  NumConnectedPlayers++;
  pthread_mutex_unlock(ServerLock);

  SendPlayerData(threadnum);

  pthread_exit(NULL);
}


// Main function which loops and continuously sends data to the player
void SendPlayerData(int threadnum)
{
  int i, j;
  int DataSize=0;
  ShipDataStruct ShipData;
  ObjDataStruct ObjData;
  unsigned char SendData[MAX_PACKET_SIZE];
  char GameStatus[MAX_OBJECTS+MAX_PLAYERS+1];

  while(!StopServer) {
    if (!ServerShips[threadnum]) {
      break;
    }

    // Send object data to player
    pthread_mutex_lock(ServerLock);
    for (i=0; i<MAX_OBJECTS; i++) {
      if (ServerObjects[i]) {
	*(SendData+DataSize)=OBJ_SEND_TYPE;
	SetupObjSendData(i, &ObjData);
	memcpy(SendData+DataSize+1, &ObjData, sizeof(ObjDataStruct));
	DataSize+=sizeof(ObjDataStruct)+1;
      }

      if (DataSize > PacketSize) {
	TransferData(SendData, &DataSize, threadnum);
      }
    }

    // Send ship data to player
    for (i=0; i<MaxPlayers; i++) {
      if (ServerShips[i]) {
	if (ServerShips[i]->Lives) {
	  *(SendData+DataSize)=SHIP_SEND_TYPE;
	  SetupShipSendData(i, &ShipData);
	  memcpy(SendData+DataSize+1, &ShipData, sizeof(ShipDataStruct));
	  DataSize+=sizeof(ShipDataStruct)+1;
	}
      }

      if (DataSize > PacketSize) {
	TransferData(SendData, &DataSize, threadnum);
      }
    }

    // Send sound data to players
    for (i=0; i<MAX_SOUNDS; i++) {
      if (Sounds[threadnum][i].type) {
	*(SendData+DataSize)=SOUND_SEND_TYPE;
	memcpy(SendData+DataSize+1, &Sounds[threadnum][i],
	       sizeof(SoundStruct));
	Sounds[threadnum][i].type=0;
	DataSize+=sizeof(SoundStruct)+1;
      }

      if (DataSize > PacketSize) {
	TransferData(SendData, &DataSize, threadnum);
      }
    }

    // Send general status update to player
    SetupStatusSendData(GameStatus, threadnum);
    *(SendData+DataSize)=STATUS_SEND_TYPE;
    memcpy(SendData+DataSize+1, GameStatus, MAX_OBJECTS+MAX_PLAYERS+1);
    DataSize+=MAX_OBJECTS+MAX_PLAYERS+1+1;
    TransferData(SendData, &DataSize, threadnum);
    pthread_mutex_unlock(ServerLock);

    usleep(NetSleepTime);
  }
}


void SetupShipSendData(int playernum, ShipDataStruct *ShipData)
{
  ShipData->x=htonl((long)ServerShips[playernum]->x);
  ShipData->y=htonl((long)ServerShips[playernum]->y);
  ShipData->phase=htons(ServerShips[playernum]->Phase);
  ShipData->playernum=playernum;
  ShipData->TripleShot=ServerShips[playernum]->TripleShot;
  ShipData->LuckOfTheIrish=ServerShips[playernum]->LuckOfTheIrish;
  ShipData->AutoFire=ServerShips[playernum]->AutoFire;
  ShipData->LongFire=ServerShips[playernum]->LongFire;
  ShipData->Speedup=ServerShips[playernum]->Speedup;
  ShipData->lives=ServerShips[playernum]->Lives;
  ShipData->score=htonl((long)ServerShips[playernum]->Score);
  ShipData->ShieldValue=htons(ServerShips[playernum]->ShieldValue);
  strcpy(ShipData->PlayerName, Connections[playernum].PlayerName);
  if (ServerShips[playernum]->ShieldOn || ServerShips[playernum]->ShieldTime) {
    ShipData->ShieldOn=1;
  } else {
    ShipData->ShieldOn=0;
  }
  if (ServerShips[playernum]->DeadTime) {
    ShipData->alive=0;
  } else {
    ShipData->alive=1;
  }
}


void SetupObjSendData(int objnum, ObjDataStruct *ObjData)
{
  ObjData->x=htonl((long)ServerObjects[objnum]->x);
  ObjData->y=htonl((long)ServerObjects[objnum]->y);
  ObjData->objnum=htons((short)objnum);
  ObjData->type=ServerObjects[objnum]->type;
}


/*
  Function to send the status of each object and player, whether they're alive
  or dead
*/
void SetupStatusSendData(char *GameStatus, int playernum)
{
  int i, j;

  for (i=0; i<MAX_OBJECTS; i++) {
    if (ServerObjects[i]) {
      GameStatus[i]=1;
    } else {
      GameStatus[i]=0;
    }
  }
  for (i=0; i<MAX_PLAYERS; i++) {
    if (ServerShips[i]) {
      if (!ServerShips[i]->DeadTime) {
	GameStatus[i+MAX_OBJECTS]=1;
      } else {
	GameStatus[i+MAX_OBJECTS]=0;
      }
    } else {
      GameStatus[i+MAX_OBJECTS]=-1;
    }
  }
  // Status of player that data is sent to
  if (ServerShips[playernum]->Lives) {
    GameStatus[MAX_PLAYERS+MAX_OBJECTS]=1;
  } else {
    GameStatus[MAX_PLAYERS+MAX_OBJECTS]=0;
  }
}


void TransferData(unsigned char *SendData, int *DataSize, int threadnum)
{
  int rc;

  if ((rc=sendto(listenfd, SendData, *DataSize, 0,
		 (struct sockaddr *)&Connections[threadnum].PlayerIp,
		 sizeof(struct sockaddr))) < 0) {
    printf("write error\n");
    /*    if (MaxPlayers==1) {
	  StopServer=1;
	  }
	  if (ServerShips[threadnum]) {
	  pthread_mutex_lock(ServerLock);
	  DeletePlayer(threadnum);
	  pthread_mutex_unlock(ServerLock);
	  }
	  pthread_exit(NULL);*/
  }
  *DataSize=0;
}
