#include <stdlib.h>
#include <unistd.h>
#include <GL/glut.h>


#include "globals.h"
#include "sizes.h"
#include "input.h"
#include "netvariables.h"
#include "game.h"
#include "screen.h"
#include "radar.h"
#include "clientconfig.h"
#include "serverconfig.h"
#include "score.h"
#include "sound.h"
#include "thread.h"
#include "netconfig.h"
#include "serverdata.h"
#include "serverobject.h"
#include "objdata.h"
#include "scorevals.h"
#include "server.h"


void MainMenu();
void SetupMainMenuNames();
void SetupMainDisplay();
void ConfMenu();
void SetupConfMenuNames();
void SetupConfDisplay();
void ServMenu();
void SetupServMenuNames();
void SetupServDisplay();
void ScoreMenu();
void SetKeyValue();
void DrawMainMenu();
void DrawConfMenu();
void DrawServMenu();
void DrawScoreMenu();
void MainMenuItemSelected();
void ConfMenuItemSelected();
void ServMenuItemSelected();
void ScoreMenuItemSelected();
void TextEntry();
void ScoreEntry();
void IntTextEntry();
void SetKey(char *KeyId, char *Key);
void SetText(char *TextId, char *Text, int MaxLength);
void SetIntText(char *TextId, int *Value, int MaxLength);
void SetServIntText(char *TextId, int *Value, int MaxLength);


int MainMenuItemHighlighted;
int ConfMenuItemHighlighted;
int ServMenuItemHighlighted;
int ScoreMenuItemHighlighted;
char *KeyValue;
char KeyName[MAX_NAME_LENGTH];
char *TextValue;
char TextName[MAX_NAME_LENGTH];
int MaxTextLength;
int *IntValue;
int NewScreenWidth, NewScreenHeight;
pthread_t ServerThread;
int ServerStarted;
int TopItemDisplayed;
int BottomItemDisplayed;
char MenuItemNames[MAX_MENU_ITEMS][2][40];
int MenuStartHeight;


void MainMenu()
{
  ClearScreen();
  DrawMainMenu();
  UpdateScreen();
}


void ConfMenu()
{
  ClearScreen();
  DrawConfMenu();
  UpdateScreen();
}


void ServMenu()
{
  ClearScreen();
  DrawServMenu();
  UpdateScreen();
}


void ScoreMenu()
{
  ClearScreen();
  DrawScoreMenu();
  UpdateScreen();
}


// Function to handle reading in one key
void SetKeyValue()
{
  char KeyText[80]="Enter key for ";

  strcat(KeyText, KeyName);

  ClearScreen();
  DrawText(ScreenWidth/2-100, ScreenHeight/2, KeyText, Red, 1);
  UpdateScreen();
}


// Function to handle reading in text entry
void TextEntry()
{
  int pos;

  ClearScreen();
  pos=DrawText(ScreenWidth/2-200, ScreenHeight/2, TextName, Blue, 1);
  DrawRect(pos+17, ScreenHeight/2-2, pos+17, ScreenHeight/2+20,
	   pos+300, ScreenHeight/2+20, pos+300, ScreenHeight/2-2, DarkGray);
  pos=DrawText(pos+20, ScreenHeight/2, TextValue, Red, 1);
  DrawRect(pos+2, ScreenHeight/2, pos+2, ScreenHeight/2+15,
	   pos+8, ScreenHeight/2+15, pos+8, ScreenHeight/2, White);
  UpdateScreen();
}


// Function to handle reading in entry of high score name
void ScoreEntry()
{
  int pos;

  ClearScreen();
  DrawText(ScreenWidth/2-200, ScreenHeight/2+50,
	   "Congratulations, you've made a high score", Blue, 1);
  pos=DrawText(ScreenWidth/2-200, ScreenHeight/2, "Enter Name", Blue, 1);
  DrawRect(pos+17, ScreenHeight/2-2, pos+17, ScreenHeight/2+20,
	   pos+300, ScreenHeight/2+20, pos+300, ScreenHeight/2-2, DarkGray);
  pos=DrawText(pos+20, ScreenHeight/2, TextValue, Red, 1);
  DrawRect(pos+2, ScreenHeight/2, pos+2, ScreenHeight/2+15,
	   pos+8, ScreenHeight/2+15, pos+8, ScreenHeight/2, White);
  UpdateScreen();
}


// Function to handle reading in text entry of integer
void IntTextEntry()
{
  int pos;
  char Value[80];

  sprintf(Value, "%d", *IntValue);

  ClearScreen();
  pos=DrawText(ScreenWidth/2-200, ScreenHeight/2, TextName, Blue, 1);
  DrawRect(pos+17, ScreenHeight/2-2, pos+17, ScreenHeight/2+20,
	   pos+300, ScreenHeight/2+20, pos+300, ScreenHeight/2-2, DarkGray);
  pos+=20;
  if (*IntValue) {
    pos=DrawText(pos, ScreenHeight/2, Value, Red, 1);
  }
  DrawRect(pos+2, ScreenHeight/2, pos+2, ScreenHeight/2+15,
	   pos+8, ScreenHeight/2+15, pos+8, ScreenHeight/2, White);
  UpdateScreen();
}


void DrawMainMenu()
{
  int i;

  for (i=TopItemDisplayed; i<=BottomItemDisplayed; i++) {
    if (i==MainMenuItemHighlighted) {
      DrawText(ScreenWidth/2-100,
	       ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
	       MenuItemNames[i][0], White, 1);
    } else {
      DrawText(ScreenWidth/2-100,
	       ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
	       MenuItemNames[i][0], Blue, 1);
    }
  }
}


void SetupMainMenuNames()
{
  strcpy(MenuItemNames[0][0], "Start Single Player Game");
  strcpy(MenuItemNames[1][0], "Start Multi Player Game");
  strcpy(MenuItemNames[2][0], "Start ATR Server");
  strcpy(MenuItemNames[3][0], "Stop ATR Server");
  strcpy(MenuItemNames[4][0], "Edit Game Configuration");
  strcpy(MenuItemNames[5][0], "Edit Server Configuration");
  strcpy(MenuItemNames[6][0], "View High Scores");
  strcpy(MenuItemNames[7][0], "Quit Game");

  if (NUM_MAINMENU_ITEMS*25 <= ScreenHeight-25) {
    MenuStartHeight=(ScreenHeight-(NUM_MAINMENU_ITEMS+1)*25+25)/2;
  } else {
    MenuStartHeight=25+25/2;
  }

  SetupMainDisplay();
}


void SetupMainDisplay()
{
  if (TopItemDisplayed > MainMenuItemHighlighted) {
    TopItemDisplayed=MainMenuItemHighlighted;
  }
  BottomItemDisplayed=TopItemDisplayed+(ScreenHeight-50)/25;
  if (BottomItemDisplayed >= NUM_MAINMENU_ITEMS) {
    BottomItemDisplayed=NUM_MAINMENU_ITEMS-1;
    TopItemDisplayed=BottomItemDisplayed-(ScreenHeight-50)/25;
    if (TopItemDisplayed < 0) {
      TopItemDisplayed=0;
    }
  }
  if (BottomItemDisplayed < MainMenuItemHighlighted) {
    TopItemDisplayed+=MainMenuItemHighlighted-BottomItemDisplayed;
    BottomItemDisplayed=MainMenuItemHighlighted;
  }
}


void DrawConfMenu()
{
  int i, pos;

  for (i=TopItemDisplayed; i<=BottomItemDisplayed; i++) {
    if (i==ConfMenuItemHighlighted) {
      pos=DrawText(ScreenWidth/2-200,
		   ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
		   MenuItemNames[i][0], White, 1);
      DrawText(pos+20, ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
	       MenuItemNames[i][1], Red, 1);
    } else {
      pos=DrawText(ScreenWidth/2-200,
		   ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
		   MenuItemNames[i][0], Blue, 1);
      DrawText(pos+20, ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
	       MenuItemNames[i][1], Red, 1);
    }
  }
}


void SetupConfMenuNames()
{
  strcpy(MenuItemNames[0][0], "Server IP (MultiPlayer Only)");
  sprintf(MenuItemNames[0][1], "%s", ServerIP);
  strcpy(MenuItemNames[1][0], "Server Port (MultiPlayer Only)");
  sprintf(MenuItemNames[1][1], "%d", ServerPort);
  strcpy(MenuItemNames[2][0], "Player Name (MultiPlayer Only)");
  sprintf(MenuItemNames[2][1], "%s", PlayerName);
  strcpy(MenuItemNames[3][0], "Screen Width");
  sprintf(MenuItemNames[3][1], "%d", NewScreenWidth);
  strcpy(MenuItemNames[4][0], "Screen Height");
  sprintf(MenuItemNames[4][1], "%d", NewScreenHeight);
  strcpy(MenuItemNames[5][0], "Radar Width (MultiPlayer Only)");
  sprintf(MenuItemNames[5][1], "%d", RadarWidth);
  strcpy(MenuItemNames[6][0], "Radar Height (MultiPlayer Only)");
  sprintf(MenuItemNames[6][1], "%d", RadarHeight);
  strcpy(MenuItemNames[7][0], "Thrust Key");
  sprintf(MenuItemNames[7][1], "%c", controls.Thrust);
  strcpy(MenuItemNames[8][0], "Left Key");
  sprintf(MenuItemNames[8][1], "%c", controls.Left);
  strcpy(MenuItemNames[9][0], "Right Key");
  sprintf(MenuItemNames[9][1], "%c", controls.Right);
  strcpy(MenuItemNames[10][0], "Fire Key");
  sprintf(MenuItemNames[10][1], "%c", controls.Fire);
  strcpy(MenuItemNames[11][0], "Shield Key");
  sprintf(MenuItemNames[11][1], "%c", controls.Shield);
  strcpy(MenuItemNames[12][0], "Score Key");
  sprintf(MenuItemNames[12][1], "%c", controls.Score);
  strcpy(MenuItemNames[13][0], "Quit Key");
  sprintf(MenuItemNames[13][1], "%c", controls.Quit);
  strcpy(MenuItemNames[14][0], "Save Settings As Defaults");
  MenuItemNames[14][1][0]='\0';
  strcpy(MenuItemNames[15][0], "Return To Main Menu");
  MenuItemNames[15][1][0]='\0';

  if (NUM_CONFMENU_ITEMS*25 <= ScreenHeight-25) {
    MenuStartHeight=(ScreenHeight-NUM_CONFMENU_ITEMS*25+50)/2;
  } else {
    MenuStartHeight=25+25/2;
  }

  SetupConfDisplay();
}


void SetupConfDisplay()
{
  if (TopItemDisplayed > ConfMenuItemHighlighted) {
    TopItemDisplayed=ConfMenuItemHighlighted;
  }
  BottomItemDisplayed=TopItemDisplayed+(ScreenHeight-50)/25;
  if (BottomItemDisplayed >= NUM_CONFMENU_ITEMS) {
    BottomItemDisplayed=NUM_CONFMENU_ITEMS-1;
    TopItemDisplayed=BottomItemDisplayed-(ScreenHeight-50)/25;
    if (TopItemDisplayed < 0) {
      TopItemDisplayed=0;
    }
  }
  if (BottomItemDisplayed < ConfMenuItemHighlighted) {
    TopItemDisplayed+=ConfMenuItemHighlighted-BottomItemDisplayed;
    BottomItemDisplayed=ConfMenuItemHighlighted;
  }
}


void DrawServMenu()
{
  int i, pos;

  for (i=TopItemDisplayed; i<=BottomItemDisplayed; i++) {
    if (i==ServMenuItemHighlighted) {
      pos=DrawText(ScreenWidth/2-200,
		   ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
		   MenuItemNames[i][0], White, 1);
      DrawText(pos+20, ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
	       MenuItemNames[i][1], Red, 1);
    } else {
      pos=DrawText(ScreenWidth/2-200,
		   ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
		   MenuItemNames[i][0], Blue, 1);
      DrawText(pos+20, ScreenHeight-MenuStartHeight-25*(i-TopItemDisplayed),
	       MenuItemNames[i][1], Red, 1);
    }
  }
}


void SetupServMenuNames()
{
  strcpy(MenuItemNames[0][0], "Maximum Number Of Players");
  sprintf(MenuItemNames[0][1], "%d", MaxPlayers);
  strcpy(MenuItemNames[1][0], "Game Type");
  sprintf(MenuItemNames[1][1], "%d", ServerGameType);
  strcpy(MenuItemNames[2][0], "World Width");
  sprintf(MenuItemNames[2][1], "%d", ServerWorldWidth);
  strcpy(MenuItemNames[3][0], "World Height");
  sprintf(MenuItemNames[3][1], "%d", ServerWorldHeight);
  strcpy(MenuItemNames[4][0], "Game Port");
  sprintf(MenuItemNames[4][1], "%d", GamePort);
  strcpy(MenuItemNames[5][0], "Shot Speed");
  sprintf(MenuItemNames[5][1], "%d", ShotSpeed);
  strcpy(MenuItemNames[6][0], "Shot Keep Alive Time");
  sprintf(MenuItemNames[6][1], "%d", ShotKeepAlive);
  strcpy(MenuItemNames[7][0], "Ship Dead Time");
  sprintf(MenuItemNames[7][1], "%d", ShipDeadTime);
  strcpy(MenuItemNames[8][0], "Start Lives");
  sprintf(MenuItemNames[8][1], "%d", StartLives);
  strcpy(MenuItemNames[9][0], "Max Kills");
  sprintf(MenuItemNames[9][1], "%d", MaxKills);
  strcpy(MenuItemNames[10][0], "Game Sleep");
  sprintf(MenuItemNames[10][1], "%d", GameSleep);
  strcpy(MenuItemNames[11][0], "Max Shield");
  sprintf(MenuItemNames[11][1], "%d", ServerMaxShield);
  strcpy(MenuItemNames[12][0], "Max Speed");
  sprintf(MenuItemNames[12][1], "%d", MaxSpeed);
  strcpy(MenuItemNames[13][0], "Start Shield");
  sprintf(MenuItemNames[13][1], "%d", StartShield);
  strcpy(MenuItemNames[14][0], "Time To Add Rock");
  sprintf(MenuItemNames[14][1], "%d", NewRockTime);
  strcpy(MenuItemNames[15][0], "Rock Time Speedup");
  sprintf(MenuItemNames[15][1], "%d", RockTimeSpeedup);
  strcpy(MenuItemNames[16][0], "Min Rock Time");
  sprintf(MenuItemNames[16][1], "%d", MinRockTime);
  strcpy(MenuItemNames[17][0], "Gravity Strength");
  sprintf(MenuItemNames[17][1], "%d", GravityStrength);
  strcpy(MenuItemNames[18][0], "Number Of Vortexes At Startup");
  sprintf(MenuItemNames[18][1], "%d", StartVortex);
  strcpy(MenuItemNames[19][0], "Number Of Rocks At Startup");
  sprintf(MenuItemNames[19][1], "%d", StartRocks);
  strcpy(MenuItemNames[20][0], "Points For Ship");
  sprintf(MenuItemNames[20][1], "%d", ShipScore);
  strcpy(MenuItemNames[21][0], "Points Subtracted For Dying");
  sprintf(MenuItemNames[21][1], "%d", ShipDeadScore);
  strcpy(MenuItemNames[22][0], "Points For Big Rock");
  sprintf(MenuItemNames[22][1], "%d", BigRockScore);
  strcpy(MenuItemNames[23][0], "Points For Medium Rock");
  sprintf(MenuItemNames[23][1], "%d", MediumRockScore);
  strcpy(MenuItemNames[24][0], "Points For Small Rock");
  sprintf(MenuItemNames[24][1], "%d", SmallRockScore);
  strcpy(MenuItemNames[25][0], "Points For Steel Roid");
  sprintf(MenuItemNames[25][1], "%d", SteelRoidScore);
  strcpy(MenuItemNames[26][0], "Time To Add Vortex");
  sprintf(MenuItemNames[26][1], "%d", VortexTime);
  strcpy(MenuItemNames[27][0], "Randomness For Vortex Time");
  sprintf(MenuItemNames[27][1], "%d", VortexRandomTime);
  strcpy(MenuItemNames[28][0], "Vortex Alive Time");
  sprintf(MenuItemNames[28][1], "%d", VortexAliveTime);
  strcpy(MenuItemNames[29][0], "Time To Add Prize");
  sprintf(MenuItemNames[29][1], "%d", PrizeTime);
  strcpy(MenuItemNames[30][0], "Randomness For Prize Time");
  sprintf(MenuItemNames[30][1], "%d", PrizeRandomTime);
  strcpy(MenuItemNames[31][0], "Prize Alive Time");
  sprintf(MenuItemNames[31][1], "%d", PrizeAliveTime);
  strcpy(MenuItemNames[32][0], "Steel Roid Hit Points");
  sprintf(MenuItemNames[32][1], "%d", SteelRoidHitPoints);
  strcpy(MenuItemNames[33][0], "Odds Of New Steel Roid");
  sprintf(MenuItemNames[33][1], "%d", SteelRoidOdds);
  strcpy(MenuItemNames[34][0], "Packet Size");
  sprintf(MenuItemNames[34][1], "%d", PacketSize);
  strcpy(MenuItemNames[35][0], "Time Between Net Updates");
  sprintf(MenuItemNames[35][1], "%d", NetSleepTime);
  strcpy(MenuItemNames[36][0], "Save Settings As Defaults");
  MenuItemNames[36][1][0]='\0';
  strcpy(MenuItemNames[37][0], "Return To Main Menu");
  MenuItemNames[37][1][0]='\0';

  if (NUM_SERVMENU_ITEMS*25 <= ScreenHeight-25) {
    MenuStartHeight=(ScreenHeight-NUM_SERVMENU_ITEMS*25+50)/2;
  } else {
    MenuStartHeight=25+25/2;
  }

  SetupServDisplay();
}


void SetupServDisplay()
{
  if (TopItemDisplayed > ServMenuItemHighlighted) {
    TopItemDisplayed=ServMenuItemHighlighted;
  }
  BottomItemDisplayed=TopItemDisplayed+(ScreenHeight-50)/25;
  if (BottomItemDisplayed >= NUM_SERVMENU_ITEMS) {
    BottomItemDisplayed=NUM_SERVMENU_ITEMS-1;
    TopItemDisplayed=BottomItemDisplayed-(ScreenHeight-50)/25;
    if (TopItemDisplayed < 0) {
      TopItemDisplayed=0;
    }
  }
  if (BottomItemDisplayed < ServMenuItemHighlighted) {
    TopItemDisplayed+=ServMenuItemHighlighted-BottomItemDisplayed;
    BottomItemDisplayed=ServMenuItemHighlighted;
  }
}


void DrawScoreMenu()
{
  int i;
  char MenuItemNames[NUM_SCOREMENU_ITEMS][40]={ "Clear High Scores",
						"Return To Main Menu" };
  char Score[10];
  int NumScoresDisplayed;

  DrawText(20, ScreenHeight-50, "Name", Blue, 1);
  DrawText(ScreenWidth/2, ScreenHeight-50, "Score", Blue, 1);

  NumScoresDisplayed=(ScreenHeight-200)/25;
  if (NumScoresDisplayed > NUM_HIGHSCORES) {
    NumScoresDisplayed=NUM_HIGHSCORES;
  }

  for (i=0; i<NumScoresDisplayed; i++) {
    sprintf(Score, "%d", HighScores[i].Score);
    DrawText(20, ScreenHeight-90-25*i, HighScores[i].Name, Red, 1);
    DrawText(ScreenWidth/2, ScreenHeight-90-25*i, Score, Red, 1);
  }

  for (i=0; i<NUM_SCOREMENU_ITEMS; i++) {
    if (i==ScoreMenuItemHighlighted) {
      DrawText(ScreenWidth/2-100, 75-25*i,
	       MenuItemNames[i], White, 1);
    } else {
      DrawText(ScreenWidth/2-100, 75-25*i,
	       MenuItemNames[i], Blue, 1);
    }
  }
}


// Handle item selected from main menu
void MainMenuItemSelected()
{
  pthread_attr_t attr;

  switch (MainMenuItemHighlighted) {
  case 0:
    if (!ServerStarted) {
      playsound(GotPrize);
      glutIdleFunc(RunGame);
      glutDisplayFunc(RunGame);
      glutKeyboardFunc(KeyDown);
      glutKeyboardUpFunc(KeyUp);
      strcpy(ServerIP, "127.0.0.1");
      ServerPort=12345;

      pthread_attr_init(&attr);
      pthread_attr_setdetachstate(&attr, !PTHREAD_CREATE_DETACHED);
      if (pthread_create(&ServerThread, &attr, Server, NULL)) {
	printf("Pthread error\n");
	exit(1);
      }
      pthread_attr_destroy(&attr);

      ServerStarted=1;
      sleep(1);
      NewGame();
    } else {
      playsound(ErrorSound);
    }
    break;
  case 1:
    playsound(GotPrize);
    glutKeyboardFunc(KeyDown);
    glutKeyboardUpFunc(KeyUp);
    glutIdleFunc(RunGame);
    glutDisplayFunc(RunGame);
    NewGame();
    break;
  case 2:
    if (!ServerStarted) {
      playsound(GotPrize);

      pthread_attr_init(&attr);
      pthread_attr_setdetachstate(&attr, !PTHREAD_CREATE_DETACHED);
      if (pthread_create(&ServerThread, &attr, Server, (void *)1)) {
	printf("Pthread error\n");
	exit(1);
      }
      pthread_attr_destroy(&attr);

      ServerStarted=1;
      sleep(1);
    } else {
      playsound(ErrorSound);
    }
    break;
  case 3:
    if (ServerStarted) {
      StopServer=1;
      if ((svrsock=socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	printf("Socket Error\n");
	exit(1);
      }

      memset(&servaddr, 0, sizeof(servaddr));

      servaddr.sin_family=AF_INET;
      servaddr.sin_port=htons(GamePort);

#ifdef CYGWIN
      servaddr.sin_addr.s_addr=inet_addr("127.0.0.1");
#else
      if (inet_pton(AF_INET, "127.0.0.1", &servaddr.sin_addr) <=0) {
	printf("inet_pton error for %s\n", ServerIP);
	exit(2);
      }
#endif

      sendto(svrsock, NULL, 0, 0, (struct sockaddr *)&servaddr,
	     sizeof(servaddr));
      close(svrsock);

      pthread_join(ServerThread, NULL);
    } else {
      playsound(ErrorSound);
    }
    break;
  case 4:
    glutKeyboardFunc(ConfMenuKey);
    glutSpecialFunc(ConfMenuSpecialKey);
    glutIdleFunc(NULL);
    glutDisplayFunc(ConfMenu);
    SetupConfMenuNames();
    ConfMenu();
    break;
  case 5:
    glutKeyboardFunc(ServMenuKey);
    glutSpecialFunc(ServMenuSpecialKey);
    glutIdleFunc(NULL);
    glutDisplayFunc(ServMenu);
    SetupServMenuNames();
    ServMenu();
    break;
  case 6:
    glutKeyboardFunc(ScoreMenuKey);
    glutSpecialFunc(ScoreMenuSpecialKey);
    glutIdleFunc(NULL);
    glutDisplayFunc(ScoreMenu);
    ScoreMenu();
    break;
  case 7:
    exit(0);
    break;
  }
}


void ConfMenuItemSelected()
{
  switch (ConfMenuItemHighlighted) {
  case 0:
    SetText("Enter Server IP", ServerIP, 15);
    break;
  case 1:
    SetIntText("Enter Server Port", &ServerPort, 5);
    break;
  case 2:
    SetText("Enter Player Name", PlayerName, MAX_PLAYERNAME_LENGTH-1);
    break;
  case 3:
    SetIntText("Enter Screen Width", &NewScreenWidth, 4);
    break;
  case 4:
    SetIntText("Enter Screen Height", &NewScreenHeight, 4);
    break;
  case 5:
    SetIntText("Enter Radar Width", &RadarWidth, 4);
    break;
  case 6:
    SetIntText("Enter Radar Height", &RadarHeight, 4);
    break;
  case 7:
    SetKey("Thrust Key", &controls.Thrust);
    break;
  case 8:
    SetKey("Left Key", &controls.Left);
    break;
  case 9:
    SetKey("Right Key", &controls.Right);
    break;
  case 10:
    SetKey("Fire Key", &controls.Fire);
    break;
  case 11:
    SetKey("Shield Key", &controls.Shield);
    break;
  case 12:
    SetKey("Score Key", &controls.Score);
    break;
  case 13:
    SetKey("Quit Key", &controls.Quit);
    break;
  case 14:
    SaveConfig();
    break;
  case 15:
    glutKeyboardFunc(MainMenuKey);
    glutSpecialFunc(MainMenuSpecialKey);
    glutIdleFunc(NULL);
    glutDisplayFunc(MainMenu);
    SetupMainMenuNames();
    MainMenu();
    break;
  }
}


void ServMenuItemSelected()
{
  switch (ServMenuItemHighlighted) {
  case 0:
    SetServIntText("Enter Maximum Number Of Players", &MaxPlayers, 4);
    break;
  case 1:
    SetServIntText("Enter Game Type", &ServerGameType, 2);
    break;
  case 2:
    SetServIntText("Enter World Width", &ServerWorldWidth, 5);
    break;
  case 3:
    SetServIntText("Enter World Height", &ServerWorldHeight, 5);
    break;
  case 4:
    SetServIntText("Enter Game Port", &GamePort, 5);
    break;
  case 5:
    SetServIntText("Enter Shot Speed", &ShotSpeed, 4);
    break;
  case 6:
    SetServIntText("Enter Shot Keep Alive Time", &ShotKeepAlive, 6);
    break;
  case 7:
    SetServIntText("Enter Ship Dead Time", &ShipDeadTime, 6);
    break;
  case 8:
    SetServIntText("Enter Start Lives", &StartLives, 4);
    break;
  case 9:
    SetServIntText("Enter Max Kills", &MaxKills, 4);
    break;
  case 10:
    SetServIntText("Enter Game Sleep", &GameSleep, 4);
    break;
  case 11:
    SetServIntText("Enter Max Shield", &ServerMaxShield, 6);
    break;
  case 12:
    SetServIntText("Enter Max Speed", &MaxSpeed, 4);
    break;
  case 13:
    SetServIntText("Enter Start Shield", &StartShield, 6);
    break;
  case 14:
    SetServIntText("Enter Time To Add Rock", &NewRockTime, 6);
    break;
  case 15:
    SetServIntText("Enter Rock Time Speedup", &RockTimeSpeedup, 6);
    break;
  case 16:
    SetServIntText("Enter Min Rock Time", &MinRockTime, 6);
    break;
  case 17:
    SetServIntText("Enter Gravity Strength", &GravityStrength, 6);
    break;
  case 18:
    SetServIntText("Enter Number Of Vortexes At Startup", &StartVortex, 4);
    break;
  case 19:
    SetServIntText("Enter Number Of Rocks At Startup", &StartRocks, 4);
    break;
  case 20:
    SetServIntText("Enter Points For Ship", &ShipScore, 6);
    break;
  case 21:
    SetServIntText("Enter Points Subtracted For Dying", &ShipDeadScore, 6);
    break;
  case 22:
    SetServIntText("Enter Points For Big Rock", &BigRockScore, 6);
    break;
  case 23:
    SetServIntText("Enter Points For Medium Rock", &MediumRockScore, 6);
    break;
  case 24:
    SetServIntText("Enter Points For Small Rock", &SmallRockScore, 6);
    break;
  case 25:
    SetServIntText("Enter Points For Steel Roid", &SteelRoidScore, 6);
    break;
  case 26:
    SetServIntText("Enter Time To Add Vortex", &VortexTime, 6);
    break;
  case 27:
    SetServIntText("Enter Randomness For Vortex Time", &VortexRandomTime, 6);
    break;
  case 28:
    SetServIntText("Enter Vortex Alive Time", &VortexAliveTime, 6);
    break;
  case 29:
    SetServIntText("Enter Time To Add Prize", &PrizeTime, 6);
    break;
  case 30:
    SetServIntText("Enter Randomness For Prize Time", &PrizeRandomTime, 6);
    break;
  case 31:
    SetServIntText("Enter Prize Alive Time", &PrizeAliveTime, 6);
    break;
  case 32:
    SetServIntText("Enter Steel Roid Hit Points", &SteelRoidHitPoints, 4);
    break;
  case 33:
    SetServIntText("Enter Odds Of New Steel Roid", &SteelRoidOdds, 4);
    break;
  case 34:
    SetServIntText("Enter Packet Size", &PacketSize, 6);
    break;
  case 35:
    SetServIntText("Enter Time Between Net Updates", &NetSleepTime, 4);
    break;
  case 36:
    SaveServerConfig();
    break;
  case 37:
    glutKeyboardFunc(MainMenuKey);
    glutSpecialFunc(MainMenuSpecialKey);
    glutIdleFunc(NULL);
    glutDisplayFunc(MainMenu);
    SetupMainMenuNames();
    MainMenu();
    break;
  }
}


void ScoreMenuItemSelected()
{
  switch (ScoreMenuItemHighlighted) {
  case 0:
    InitHighScores();
    SaveHighScores();
    ScoreMenu();
    break;
  case 1:
    glutKeyboardFunc(MainMenuKey);
    glutSpecialFunc(MainMenuSpecialKey);
    glutIdleFunc(NULL);
    glutDisplayFunc(MainMenu);
    SetupMainMenuNames();
    MainMenu();
    break;
  }
}


void SetKey(char *KeyId, char *Key)
{
  strcpy(KeyName, KeyId);
  KeyValue=Key;

  glutKeyboardFunc(SetKeyValueKey);
  glutSpecialFunc(NULL);
  glutIdleFunc(NULL);
  glutDisplayFunc(SetKeyValue);
  SetKeyValue();
}


void SetText(char *TextId, char *Text, int MaxLength)
{
  strcpy(TextName, TextId);
  TextValue=Text;
  MaxTextLength=MaxLength;

  glutKeyboardFunc(TextEntryKey);
  glutSpecialFunc(NULL);
  glutIdleFunc(NULL);
  glutDisplayFunc(TextEntry);
  TextEntry();
}


void SetIntText(char *TextId, int *Value, int MaxLength)
{
  strcpy(TextName, TextId);
  IntValue=Value;
  MaxTextLength=MaxLength;

  glutKeyboardFunc(IntTextEntryKey);
  glutSpecialFunc(NULL);
  glutIdleFunc(NULL);
  glutDisplayFunc(IntTextEntry);
  IntTextEntry();
}


void SetServIntText(char *TextId, int *Value, int MaxLength)
{
  strcpy(TextName, TextId);
  IntValue=Value;
  MaxTextLength=MaxLength;

  glutKeyboardFunc(ServIntTextEntryKey);
  glutSpecialFunc(NULL);
  glutIdleFunc(NULL);
  glutDisplayFunc(IntTextEntry);
  IntTextEntry();
}
