#include "screen.h"
#include "sizes.h"


/*
  Main class for any object that appears on the screen.  The ship is a derived
  class.
*/
class Object {

public:
  long xpos, ypos;
  Sprite *sprite;
  int phase;
  char objtype;

  Object(int X, int Y, char type)
  {
    objtype=type;
    phase=0;
    SetPos(X, Y);

    // Setup sprite based on the object type
    if (objtype==SHOT_TYPE) {
      sprite = ShotSprite;
    } else if (objtype==BIGROCK_TYPE) {
      sprite = BigRockSprite;
    } else if (objtype==MEDIUMROCK_TYPE) {
      sprite = MediumRockSprite;
    } else if (objtype==SMALLROCK_TYPE) {
      sprite = SmallRockSprite;
    } else if (objtype==SHRAPNEL1_TYPE) {
      sprite = Shrapnel1Sprite;
    } else if (objtype==SHRAPNEL2_TYPE) {
      sprite = Shrapnel2Sprite;
    } else if (objtype==SHIPEXPLOSION_TYPE) {
      sprite = ShipExplosionSprite;
    } else if (objtype==EXPLOSION_TYPE) {
      sprite = ExplosionSprite;
    } else if (objtype==PRIZE_TYPE) {
      sprite = PrizeSprite;
    } else if (objtype==VORTEX_TYPE) {
      sprite = VortexSprite;
    } else if (objtype==STEELROID_TYPE) {
      sprite = SteelRoidSprite;
    }
  }


  // Set the object position.  The world is setup to be wrap around.
  void SetPos(int X, int Y)
  {
    xpos = X;
    while (xpos > WorldWidth) {
      xpos-=WorldWidth;
    }
    while (xpos < 0) {
      xpos+=WorldWidth;
    }

    ypos = Y;
    while (ypos > WorldHeight) {
      ypos-=WorldHeight;
    }
    while (ypos < 0) {
      ypos+=WorldHeight;
    }
  }


  // Queue the sprite to be displayed
  void ShowSprite(int xoffset, int yoffset)
  {
    int relxpos, relypos;

    relxpos=(xpos-xoffset)+ScreenWidth/2;
    if (relxpos < 0) {
      relxpos+=WorldWidth;
    } else if (relxpos >= WorldWidth) {
      relxpos-=WorldWidth;
    }

    relypos=(ypos-yoffset)+ScreenHeight/2;
    if (relypos < 0) {
      relypos+=WorldHeight;
    } else if (relypos >= WorldHeight) {
      relypos-=WorldHeight;
    }

    QueueSprite(relxpos, relypos, sprite, phase);

    phase++;
    if (phase>=sprite->Frames) {
      phase=0;
    }
  }
};
