#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "defines.h"
#include "thread.h"
#include "serverdata.h"
#include "servergame.h"
#include "netdata.h"
#include "netconfig.h"
#include "servership.h"
#include "mainplayerthread.h"


// Function to loop and continuously read all network data
void ReadNetworkData()
{
  int i, n, playernum, rc;
  char readval[64];
  int threadnum;
  int keypress;
  int WaitTime;
  struct sockaddr_in PlayerIp;
#ifdef CYGWIN
  int addrlen=16;
#else
  socklen_t addrlen=16;
#endif
  struct timezone tz;
  struct timeval now;
  KeyValuesStruct KeyValues;
  GameParametersStruct GameParameters;

  // Setup game parameters
  GameParameters.MaxPlayers=htons(MaxPlayers);
  GameParameters.WorldWidth=htons(ServerWorldWidth);
  GameParameters.WorldHeight=htons(ServerWorldHeight);
  GameParameters.MaxShield=htonl(ServerMaxShield);
  if (MaxPlayers==1) {
    GameParameters.GameType=htons(SINGLE_PLAYER);
  } else {
    GameParameters.GameType=htons(ServerGameType);
  }

  while (!StopServer) {
    if ((n=recvfrom(listenfd, readval, 64, 0,
		    (struct sockaddr *)&PlayerIp, &addrlen)) <= 0) {
      continue;
    }
    pthread_mutex_lock(ServerLock);

    // Check for status update packet
    if (n==sizeof(KeyValuesStruct)) {
      for (i=0; i<MAX_PLAYERS; i++) {
	if (Connections[i].Connected) {
	  if (Connections[i].PlayerIp.sin_addr.s_addr==
	      PlayerIp.sin_addr.s_addr) {
	    gettimeofday(&Connections[i].LastPacketTime, &tz);
	    memcpy(&KeyValues, readval, sizeof(KeyValuesStruct));

	    ServerShips[i]->Action.thrusting=KeyValues.ThrustKey;
	    ServerShips[i]->Action.right=KeyValues.RightKey;
	    ServerShips[i]->Action.left=KeyValues.LeftKey;
	    ServerShips[i]->Action.shield=KeyValues.ShieldKey;
	    if (ServerShips[i]->AutoFire) {
	      ServerShips[i]->Action.shooting=KeyValues.FireKey;
	    }
	  }
	}
      }
    } else if (n==MAX_PLAYERNAME_LENGTH) {
      // Check if player is already connected, if so delete them and reconnect
      for (i=0; i<MAX_PLAYERS; i++) {
	if (Connections[i].Connected) {
	  if (Connections[i].PlayerIp.sin_addr.s_addr
	      ==PlayerIp.sin_addr.s_addr) {
	    DeletePlayer(i);
	  }
	}
      }

      if (NumConnectedPlayers==MaxPlayers) {
	pthread_mutex_unlock(ServerLock);
	continue;
      }

      for (i=0; i<MAX_PLAYERS; i++) {
	if (!Connections[i].Connected) {
	  // Send Game Parameters to player and connect them to the game
	  GameParameters.PlayerNum=htons(i);
	  if (sendto(listenfd, &GameParameters, sizeof(GameParametersStruct),
		     0, (struct sockaddr *)&PlayerIp, addrlen) < 0) {
	    printf("Write Error\n");
	    break;
	  }

	  memcpy(&Connections[i].PlayerIp, &PlayerIp,
		 sizeof(struct sockaddr_in));
	  gettimeofday(&Connections[i].LastPacketTime, &tz);
	  readval[MAX_PLAYERNAME_LENGTH-1]=0;
	  strcpy(Connections[i].PlayerName, readval);

	  // Spawn off a thread for the connected player
	  if (rc = pthread_create(&ServerThreads[i], NULL,
				  MainPlayerThread, (void *)i)) {
	    printf("ERROR, return code from pthread_create() is %d\n", rc);
	    exit(8);
	  }
	  break;
	}
      }
    } else if (n==1) {

      playernum=-1;

      for (i=0; i<MAX_PLAYERS; i++) {
	if (Connections[i].Connected) {
	  if (Connections[i].PlayerIp.sin_addr.s_addr==
	      PlayerIp.sin_addr.s_addr) {
	    playernum=i;
	    break;
	  }
	}
      }

      if (playernum==-1) {
	pthread_mutex_unlock(ServerLock);
	continue;
      }

      keypress=*readval/0x10;
      if (*readval > 0x10) {
	*readval-=0x10;
      }

      if (keypress) {
	switch (*readval) {
	case THRUST_KEY:
	  ServerShips[playernum]->Action.thrusting=TRUE;
	  break;
	case RIGHT_KEY:
	  ServerShips[playernum]->Action.right=TRUE;
	  break;
	case LEFT_KEY:
	  ServerShips[playernum]->Action.left=TRUE;
	  break;
	case FIRE_KEY:
	  ServerShips[playernum]->Action.shooting=TRUE;
	  break;
	case SHIELD_KEY:
	  ServerShips[playernum]->Action.shield=TRUE;
	  break;
	case ABORT_KEY:
	  if (MaxPlayers==1) {
	    StopServer=1;
	  }
	  if ((rc=sendto(listenfd, "\0", 1, 0,
			 (struct sockaddr *)&Connections[playernum].PlayerIp,
			 sizeof(struct sockaddr))) < 0) {
	    printf("write error\n");
	  }
	  DeletePlayer(playernum);
	  break;
	default:
	  break;
	}
      } else {
	switch (*readval) {
	case THRUST_KEY:
	  ServerShips[playernum]->Action.thrusting=FALSE;
	  break;
	case RIGHT_KEY:
	  ServerShips[playernum]->Action.right=FALSE;
	  break;
	case LEFT_KEY:
	  ServerShips[playernum]->Action.left=FALSE;
	  break;
	case FIRE_KEY:
	  if (ServerShips[playernum]->AutoFire) {
	    ServerShips[playernum]->Action.shooting=FALSE;
	  }
	  break;
	case SHIELD_KEY:
	  ServerShips[playernum]->Action.shield=FALSE;
	  break;
	default:
	  break;
	}
      }
    }

    // Check if any players are timed out
    gettimeofday(&now, &tz);
    for (i=0; i<MaxPlayers; i++) {
      if (Connections[i].Connected) {
	WaitTime=(now.tv_sec-Connections[i].LastPacketTime.tv_sec)*1000000+
	  (now.tv_usec-Connections[i].LastPacketTime.tv_usec);
	if (WaitTime > 10000000) {
	  if (MaxPlayers==1) {
	    StopServer=1;
	  }
	  DeletePlayer(i);
	}
      }
    }

    pthread_mutex_unlock(ServerLock);
  }

  close(listenfd);

  ServerStarted=0;

  pthread_exit(NULL);
}
