#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "ship.h"
#include "netvariables.h"
#include "serversound.h"
#include "status.h"
#include "thread.h"
#include "objglobals.h"


void SetShipData(ShipDataStruct *ShipData);
void SetObjData(ObjDataStruct *ObjData);
void SetStatus(char *GameStatus);


/*
  Function to read the data from the server and update the game variables as
  necessary.
*/
void *ReadServerThread(void *sockfd)
{
  int n, i;
  char type;
  unsigned char readdata[MAX_PACKET_SIZE];
  ShipDataStruct ShipData;
  ObjDataStruct ObjData;
  char GameStatus[MAX_OBJECTS+MAX_PLAYERS+1];
  SoundStruct SoundData;
  unsigned char *ptr;
  struct sockaddr_in recvIP;
#ifdef CYGWIN
  int addrlen=16;
#else
  socklen_t addrlen=16;
#endif

  while (!GameStop) {
    if ((n=recvfrom(svrsock, (char *)readdata, MAX_PACKET_SIZE, 0,
		    (struct sockaddr *)&recvIP, &addrlen)) < 0) {
      continue;
    }

    if (n==1 && *readdata=='\0') {
      while (!GameStop) {
	usleep(50);
      }
      break;
    }

    // Parse through server packet for game data
    ptr=readdata;
    while (n) {
      type=*ptr;
      ptr++;
      n--;
      if (type==SHIP_SEND_TYPE) {
	if (n>=sizeof(ShipDataStruct)) {
	  memcpy(&ShipData, ptr, sizeof(ShipDataStruct));
	  SetShipData(&ShipData);
	  ptr+=sizeof(ShipDataStruct);
	  n-=sizeof(ShipDataStruct);
	} else {
	  break;
	}
      } else if (type==OBJ_SEND_TYPE) {
	if (n>=sizeof(ObjDataStruct)) {
	  memcpy(&ObjData, ptr, sizeof(ObjDataStruct));
	  SetObjData(&ObjData);
	  ptr+=sizeof(ObjDataStruct);
	  n-=sizeof(ObjDataStruct);
	} else {
	  break;
	}
      } else if (type==STATUS_SEND_TYPE) {
	if (n>=MAX_OBJECTS+MAX_PLAYERS+1) {
	  memcpy(GameStatus, ptr, MAX_OBJECTS+MAX_PLAYERS+1);
	  SetStatus(GameStatus);
	  ptr+=MAX_OBJECTS+MAX_PLAYERS+1;
	    n-=MAX_OBJECTS+MAX_PLAYERS+1;
	} else {
	  break;
	}
      } else if (type==SOUND_SEND_TYPE) {
	if (n>=sizeof(SoundStruct)) {
	  memcpy(&SoundData, ptr, sizeof(SoundStruct));
	  playsound(SoundData.type);
	  ptr+=sizeof(SoundStruct);
	  n-=sizeof(SoundStruct);
	} else {
	  break;
	}
      }
    }
  }
  pthread_mutex_lock(ClientLock);

  for (i=0; i<NumPlayers; i++) {
    delete Ships[i];
  }

  for (i=0; i<MAX_OBJECTS; i++) {
    if (Sprites[i]) {
      delete Sprites[i];
      Sprites[i]=NULL;
    }
  }

  pthread_mutex_unlock(ClientLock);

  close(svrsock);

  pthread_exit(NULL);
}


void SetShipData(ShipDataStruct *ShipData)
{
  pthread_mutex_lock(ClientLock);

  Ships[ShipData->playernum]->xpos=ntohl(ShipData->x);
  Ships[ShipData->playernum]->ypos=ntohl(ShipData->y);
  Ships[ShipData->playernum]->phase=ntohs(ShipData->phase);
  Ships[ShipData->playernum]->alive=ShipData->alive;
  Ships[ShipData->playernum]->ShieldOn=ShipData->ShieldOn;
  Ships[ShipData->playernum]->score=ntohl(ShipData->score);
  Ships[ShipData->playernum]->TripleShot=ShipData->TripleShot;
  Ships[ShipData->playernum]->LuckOfTheIrish=ShipData->LuckOfTheIrish;
  Ships[ShipData->playernum]->AutoFire=ShipData->AutoFire;
  Ships[ShipData->playernum]->LongFire=ShipData->LongFire;
  Ships[ShipData->playernum]->Speedup=ShipData->Speedup;
  strcpy(Ships[ShipData->playernum]->ShipName, ShipData->PlayerName);
  if (ShipData->playernum==myShip) {
    Lives=ShipData->lives;
    ShieldValue=ntohs(ShipData->ShieldValue);
  }

  pthread_mutex_unlock(ClientLock);
}


void SetObjData(ObjDataStruct *ObjData)
{
  pthread_mutex_lock(ClientLock);

  if (Sprites[ntohs(ObjData->objnum)]==NULL) {
    Sprites[ntohs(ObjData->objnum)]=new Object(ntohl(ObjData->x),
					       ntohl(ObjData->y),
					       ObjData->type);
  } else {
    if (ObjData->type!=Sprites[ntohs(ObjData->objnum)]->objtype) {
      delete Sprites[ntohs(ObjData->objnum)];
      Sprites[ntohs(ObjData->objnum)]=new Object(ntohl(ObjData->x),
						 ntohl(ObjData->y),
						 ObjData->type);
    } else {
      Sprites[ntohs(ObjData->objnum)]->SetPos(ntohl(ObjData->x),
					      ntohl(ObjData->y));
    }
  }

  pthread_mutex_unlock(ClientLock);
}


void SetStatus(char *GameStatus)
{
  int i;

  for (i=0; i<MAX_OBJECTS; i++) {
    if (GameStatus[i]==0 && Sprites[i]!=NULL) {
      delete Sprites[i];
      Sprites[i]=NULL;
    }
  }
  for (i=0; i<NumPlayers; i++) {
    if (GameStatus[i+MAX_OBJECTS]==1) {
      Ships[i]->alive=1;
      Ships[i]->Connected=1;
    } else if (GameStatus[i+MAX_OBJECTS]==0) {
      Ships[i]->alive=0;
      Ships[i]->Connected=1;
    } else if (GameStatus[i+MAX_OBJECTS]==-1) {
      Ships[i]->alive=0;
      Ships[i]->Connected=0;
    }
  }
  if (GameStatus[MAX_PLAYERS+MAX_OBJECTS]==0) {
    GameOver=1;
  } else {
    GameOver=0;
  }
}
