#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "defines.h"
#include "netconfig.h"
#include "serverdata.h"
#include "serverobject.h"
#include "objdata.h"
#include "scorevals.h"
#include "fileutils.h"


void ReadServerConfig(char *ServerConfigFileName);
void SetServerDefaults();
void ServerAssignValue(char *VariableName, char *value);
void SaveServerConfig();


char ServerConfigFileName[MAX_FILENAME_SIZE];
char ServerSingleConfigFileName[MAX_FILENAME_SIZE];
int GameSleep;
int ServerGameType;
int ShotKeepAlive;
int ShipDeadTime;
int StartLives;
int MaxKills;
int NumConnectedPlayers;
int ServerWorldHeight;
int ServerWorldWidth;
int GravityStrength;
int StartVortex;
int StartRocks;
int NewRockTime;
int RockTimeSpeedup;
int MinRockTime;
int ShotSpeed;
int ServerMaxShield;
int StartShield;
int MaxSpeed;
int MaxPlayers;
int ShipScore;
int ShipDeadScore;
int BigRockScore;
int MediumRockScore;
int SmallRockScore;
int SteelRoidScore;
int VortexTime;
int VortexRandomTime;
int VortexAliveTime;
int PrizeTime;
int PrizeRandomTime;
int PrizeAliveTime;
int SteelRoidOdds;
int SteelRoidHitPoints;
int PacketSize;
int NetSleepTime;


void ReadServerConfig(char *ServerConfigFileName)
{
  FILE *ConfigFile;

  SetServerDefaults();

  if (!(ConfigFile=fopen(ServerConfigFileName, "r"))) {
    return;
  }

  ReadVariables(ConfigFile, &ServerAssignValue);

  fclose(ConfigFile);
}


void SetServerDefaults()
{
  ServerWorldWidth=640;
  ServerWorldHeight=480;
  ServerGameType=1;
  GamePort=12346;
  ShotSpeed=16;
  GameSleep=5;
  ServerMaxShield=0;
  StartShield=0;
  NewRockTime=10000;
  RockTimeSpeedup=10;
  MinRockTime=1000;
  GravityStrength=100;
  StartVortex=0;
  StartRocks=3;
  BigRockScore=10;
  MediumRockScore=20;
  SmallRockScore=40;
  SteelRoidScore=100;
  ShipScore=200;
  ShipDeadScore=0;
  StartLives=3;
  MaxKills=20;
  VortexTime=0;
  VortexRandomTime=0;
  VortexAliveTime=20000;
  PrizeTime=0;
  PrizeRandomTime=0;
  PrizeAliveTime=4000;
  SteelRoidOdds=3;
  SteelRoidHitPoints=15;
  PacketSize=1024;
  ObjSizes[SHIP_TYPE]=SHIP_SIZE;
  ObjSizes[SHOT_TYPE]=SHOT_SIZE;
  ObjSizes[VORTEX_TYPE]=VORTEX_SIZE;
  ObjSizes[SMALLROCK_TYPE]=SMALLROCK_SIZE;
  ObjSizes[MEDIUMROCK_TYPE]=MEDIUMROCK_SIZE;
  ObjSizes[BIGROCK_TYPE]=BIGROCK_SIZE;
  ObjSizes[PRIZE_TYPE]=PRIZE_SIZE;
  ObjSizes[STEELROID_TYPE]=STEELROID_SIZE;
  ObjWeights[SHIP_TYPE]=SHIP_WEIGHT;
  ObjWeights[SHOT_TYPE]=SHOT_WEIGHT;
  ObjWeights[VORTEX_TYPE]=VORTEX_WEIGHT;
  ObjWeights[SMALLROCK_TYPE]=SMALLROCK_WEIGHT;
  ObjWeights[MEDIUMROCK_TYPE]=MEDIUMROCK_WEIGHT;
  ObjWeights[BIGROCK_TYPE]=BIGROCK_WEIGHT;
  ObjWeights[STEELROID_TYPE]=STEELROID_WEIGHT;
}


void ServerAssignValue(char *VariableName, char *value)
{
  int temp;

  if (!strcmp(VariableName, "MAX_NUMBER_OF_PLAYERS")) {
    MaxPlayers=atoi(value);
  } else if (!strcmp(VariableName, "GAME_TYPE")) {
    ServerGameType=atoi(value);
  } else if (!strcmp(VariableName, "WORLD_WIDTH")) {
    ServerWorldWidth=atoi(value);
  } else if (!strcmp(VariableName, "WORLD_HEIGHT")) {
    ServerWorldHeight=atoi(value);
  } else if (!strcmp(VariableName, "GAME_PORT")) {
    GamePort=atoi(value);
  } else if (!strcmp(VariableName, "SHOT_SPEED")) {
    ShotSpeed=atoi(value);
  } else if (!strcmp(VariableName, "SHOT_KEEP_ALIVE")) {
    ShotKeepAlive=atoi(value);
  } else if (!strcmp(VariableName, "SHIP_DEAD_TIME")) {
    ShipDeadTime=atoi(value);
  } else if (!strcmp(VariableName, "START_LIVES")) {
    StartLives=atoi(value);
  } else if (!strcmp(VariableName, "MAX_KILLS")) {
    MaxKills=atoi(value);
  } else if (!strcmp(VariableName, "GAME_SLEEP")) {
    GameSleep=atoi(value);
  } else if (!strcmp(VariableName, "MAX_SHIELD")) {
    ServerMaxShield=atoi(value);
  } else if (!strcmp(VariableName, "MAX_SPEED")) {
    MaxSpeed=atoi(value);
  } else if (!strcmp(VariableName, "START_SHIELD")) {
    StartShield=atoi(value);
  } else if (!strcmp(VariableName, "ROCK_TIME")) {
    NewRockTime=atoi(value);
  } else if (!strcmp(VariableName, "ROCK_TIME_SPEEDUP")) {
    RockTimeSpeedup=atoi(value);
  } else if (!strcmp(VariableName, "MIN_ROCK_TIME")) {
    MinRockTime=atoi(value);
  } else if (!strcmp(VariableName, "GRAVITY_STRENGTH")) {
    GravityStrength=atoi(value);
  } else if (!strcmp(VariableName, "START_VORTEX")) {
    StartVortex=atoi(value);
  } else if (!strcmp(VariableName, "START_ROCKS")) {
    StartRocks=atoi(value);
  } else if (!strcmp(VariableName, "SHIP_SCORE")) {
    ShipScore=atoi(value);
  } else if (!strcmp(VariableName, "SHIP_DEAD_SCORE")) {
    ShipDeadScore=atoi(value);
  } else if (!strcmp(VariableName, "BIGROCK_SCORE")) {
    BigRockScore=atoi(value);
  } else if (!strcmp(VariableName, "MEDIUMROCK_SCORE")) {
    MediumRockScore=atoi(value);
  } else if (!strcmp(VariableName, "SMALLROCK_SCORE")) {
    SmallRockScore=atoi(value);
  } else if (!strcmp(VariableName, "STEELROID_SCORE")) {
    SteelRoidScore=atoi(value);
  } else if (!strcmp(VariableName, "VORTEX_TIME")) {
    VortexTime=atoi(value);
  } else if (!strcmp(VariableName, "VORTEX_RANDOM_TIME")) {
    VortexRandomTime=atoi(value);
  } else if (!strcmp(VariableName, "VORTEX_ALIVE_TIME")) {
    VortexAliveTime=atoi(value);
  } else if (!strcmp(VariableName, "PRIZE_TIME")) {
    PrizeTime=atoi(value);
  } else if (!strcmp(VariableName, "PRIZE_RANDOM_TIME")) {
    PrizeRandomTime=atoi(value);
  } else if (!strcmp(VariableName, "PRIZE_ALIVE_TIME")) {
    PrizeAliveTime=atoi(value);
  } else if (!strcmp(VariableName, "STEELROID_HIT_POINTS")) {
    SteelRoidHitPoints=atoi(value);
  } else if (!strcmp(VariableName, "STEELROID_ODDS")) {
    SteelRoidOdds=atoi(value);
  } else if (!strcmp(VariableName, "PACKET_SIZE")) {
    PacketSize=atoi(value);
  } else if (!strcmp(VariableName, "NET_SLEEP_TIME")) {
    NetSleepTime=atoi(value);
  }
}


void SaveServerConfig()
{
  FILE *ConfigFile;

  if (!(ConfigFile=fopen(ServerConfigFileName, "w"))) {
    return;
  }

  fprintf(ConfigFile, "MAX_NUMBER_OF_PLAYERS=%d\n", MaxPlayers);
  fprintf(ConfigFile, "GAME_TYPE=%d\n", ServerGameType);
  fprintf(ConfigFile, "WORLD_WIDTH=%d\n", ServerWorldWidth);
  fprintf(ConfigFile, "WORLD_HEIGHT=%d\n", ServerWorldHeight);
  fprintf(ConfigFile, "GAME_PORT=%d\n", GamePort);
  fprintf(ConfigFile, "SHOT_SPEED=%d\n", ShotSpeed);
  fprintf(ConfigFile, "SHOT_KEEP_ALIVE=%d\n", ShotKeepAlive);
  fprintf(ConfigFile, "SHIP_DEAD_TIME=%d\n", ShipDeadTime);
  fprintf(ConfigFile, "START_LIVES=%d\n", StartLives);
  fprintf(ConfigFile, "MAX_KILLS=%d\n", MaxKills);
  fprintf(ConfigFile, "GAME_SLEEP=%d\n", GameSleep);
  fprintf(ConfigFile, "MAX_SHIELD=%d\n", ServerMaxShield);
  fprintf(ConfigFile, "MAX_SPEED=%d\n", MaxSpeed);
  fprintf(ConfigFile, "START_SHIELD=%d\n", StartShield);
  fprintf(ConfigFile, "ROCK_TIME=%d\n", NewRockTime);
  fprintf(ConfigFile, "ROCK_TIME_SPEEDUP=%d\n", RockTimeSpeedup);
  fprintf(ConfigFile, "MIN_ROCK_TIME=%d\n", MinRockTime);
  fprintf(ConfigFile, "GRAVITY_STRENGTH=%d\n", GravityStrength);
  fprintf(ConfigFile, "START_VORTEX=%d\n", StartVortex);
  fprintf(ConfigFile, "START_ROCKS=%d\n", StartRocks);
  fprintf(ConfigFile, "SHIP_SCORE=%d\n", ShipScore);
  fprintf(ConfigFile, "SHIP_DEAD_SCORE=%d\n", ShipDeadScore);
  fprintf(ConfigFile, "BIGROCK_SCORE=%d\n", BigRockScore);
  fprintf(ConfigFile, "MEDIUMROCK_SCORE=%d\n", MediumRockScore);
  fprintf(ConfigFile, "SMALLROCK_SCORE=%d\n", SmallRockScore);
  fprintf(ConfigFile, "STEELROID_SCORE=%d\n", SteelRoidScore);
  fprintf(ConfigFile, "VORTEX_TIME=%d\n", VortexTime);
  fprintf(ConfigFile, "VORTEX_RANDOM_TIME=%d\n", VortexRandomTime);
  fprintf(ConfigFile, "VORTEX_ALIVE_TIME=%d\n", VortexAliveTime);
  fprintf(ConfigFile, "PRIZE_TIME=%d\n", PrizeTime);
  fprintf(ConfigFile, "PRIZE_RANDOM_TIME=%d\n", PrizeRandomTime);
  fprintf(ConfigFile, "PRIZE_ALIVE_TIME=%d\n", PrizeAliveTime);
  fprintf(ConfigFile, "STEELROID_HIT_POINTS=%d\n", SteelRoidHitPoints);
  fprintf(ConfigFile, "STEELROID_ODDS=%d\n", SteelRoidOdds);
  fprintf(ConfigFile, "PACKET_SIZE=%d\n", PacketSize);
  fprintf(ConfigFile, "NET_SLEEP_TIME=%d\n", NetSleepTime);

  fclose(ConfigFile);
}
