#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#include "defines.h"
#include "serverdata.h"
#include "netconfig.h"
#include "servership.h"
#include "objdata.h"
#include "scorevals.h"
#include "sounddefs.h"
#include "mainplayerthread.h"
#include "thread.h"
#include "timeutils.h"


void *StartGame(void *threadid);
void DeletePlayer(int playernum);
void InitPlayerData(int playernum);
void InitObjectData();
void HandleActions(int playernum, int update);
void FireShot(int playernum);
void CheckForCollisions();
int CheckForCollision(double x1, double y1, double x2, double y2,
		      long size1, long size2);
int ShipHit(int playernum, int objnum);
void UpdateTimes();
void CalcShotVelocity(int playernum, int phaseoffset,
		      double *xvel, double *yvel);
int CreateObject(char type, double xpos, double ypos,
		 double xvel, double yvel, int AliveTime);
void CreateVortex();
void CreateRandomRock();
void RockHit(int objnum, int playernum);
int SteelRoidHit(int objnum, int playernum);
void KillSteelRoid(int objnum);
void GetPrize(int objnum, int playernum);
void PrizeHit(int objnum, int playernum);
void AddSound(int playernum, double x, double y, char type);
void AddSoundAll(double x, double y, char type);
int BounceObjects(ServerObject *obj1, ServerObject *obj2);
double CalcDistance(double x1, double y1, int size1,
		    double x2, double y2, int size2);
double CalcAngle(double x1, double y1, int size1,
		 double x2, double y2, int size2);


int EndDeathMatch;
SoundStruct Sounds[MAX_PLAYERS][MAX_SOUNDS];


// Main function to run the game
void *StartGame(void *threadid)
{
#ifdef CYGWIN
  int sleepcount=0;
#endif
  int i, j=0, k=0, l=0;
  int update, keyupdate;
  int secondupdate;
  int RockTime=0;
  int threadnum, RunTime;
  unsigned long TimeElapsed, OldTimeElapsed=0;
  int TimeCounter=0;
  int SleepTimesElapsed;
  int SaveRockTime;
  int MakePrizeTime, PrizeCounter=0;
  int MakeVortexTime, VortexCounter=0;
  int SleepTime;

  threadnum=(int)threadid;

  pthread_mutex_lock(ServerLock);

  for (i=0; i<MAX_PLAYERS; i++) {
    ServerShips[i]=NULL;
  }

  if (MaxPlayers==1) {
    ServerGameType=SINGLE_PLAYER;
  }

  EndDeathMatch=FALSE;

  MakePrizeTime=PrizeTime+(rand()%PrizeRandomTime)*2-PrizeRandomTime;
  MakeVortexTime=VortexTime+(rand()%VortexRandomTime)*2-VortexRandomTime;

  SaveRockTime=NewRockTime;

#ifndef CYGWIN
  InitTime();
#endif

  SleepTime=2000*GameSleep;

  InitObjectData();

  for (i=0; i<StartRocks; i++) {
    CreateRandomRock();
  }

  for (i=0; i<StartVortex; i++) {
    CreateVortex();
  }

  GameStart=true;
  update=FALSE;
  keyupdate=FALSE;
  secondupdate=FALSE;

  while(!StopServer) {
    // Allow other threads to access shared data while we're sleeping
    pthread_mutex_unlock(ServerLock);

#ifdef CYGWIN
    if (sleepcount != 3) {
      usleep(SleepTime);
      sleepcount++;
    } else {
      sleepcount=0;
    }
#else
    TimeElapsed=GetTimeElapsed();
    if (TimeElapsed < OldTimeElapsed) {
      OldTimeElapsed=TimeElapsed;
    }
    /*
      We need this because usleep is somewhat inaccurate and we may be
      slighly under or over the necessary sleep time
    */
    SleepTimesElapsed=(int)(TimeElapsed/SleepTime)-
                      (int)(OldTimeElapsed/SleepTime);

    if (SleepTimesElapsed < 0 || SleepTimesElapsed > 10) {
      SleepTimesElapsed=0;
    }

    TimeCounter+=1-SleepTimesElapsed;

    if (TimeCounter>0) {
      usleep(SleepTime*TimeCounter);
    }
    OldTimeElapsed=TimeElapsed;

#endif
    pthread_mutex_lock(ServerLock);

    if (NumConnectedPlayers) {
      // Update the game at a constant rate no matter how long we sleep
      k+=GameSleep;
      if (k >= 10) {
	update=TRUE;
	k-=10;
      } else {
	update=FALSE;
      }

      // Updates for key input
      l+=GameSleep;
      if (l >= 20) {
	keyupdate=TRUE;
	l-=20;
      } else {
	keyupdate=FALSE;
      }

      // Updates that we have to do once a second
      j+=GameSleep;
      if (j >= 1000) {
	secondupdate=TRUE;
	j-=1000;
      } else {
	secondupdate=FALSE;
      }

      if (NewRockTime) {
	RockTime+=GameSleep;
	if (RockTime >= NewRockTime) {
	  RockTime=0;
	  CreateRandomRock();
	}
      }

      if (PrizeTime) {
	PrizeCounter+=GameSleep;
	if (PrizeCounter >= MakePrizeTime) {
	  MakePrizeTime=PrizeTime+(rand()%PrizeRandomTime)*2-PrizeRandomTime;
	  PrizeCounter=0;
	  CreateObject(PRIZE_TYPE,
		       ((rand()%256)/256.0)*ServerWorldWidth,
		       ((rand()%256)/256.0)*ServerWorldHeight,
		       (rand()%80-40)/16.0, (rand()%80-40)/16.0,
		       PrizeAliveTime);
	}
      }

      if (VortexTime) {
	VortexCounter+=GameSleep;
	if (VortexCounter >= MakeVortexTime) {
	  MakeVortexTime=VortexTime+(rand()%VortexRandomTime)*2-
	                 VortexRandomTime;
	  VortexCounter=0;
	  CreateVortex();
	}
      }

      for (i=0; i<MaxPlayers; i++) {
	if (ServerShips[i]) {
	  if (ServerShips[i]->Lives) {
	    if (!ServerShips[i]->DeadTime) {
	      HandleActions(i, keyupdate);
	    } else {
	      ServerShips[i]->Action.shooting=FALSE;
	    }
	  } else {
	    if (ServerShips[i]->Action.shooting) {
	      // Start new game
	      InitPlayerData(i);
	      if (ServerGameType==SINGLE_PLAYER) {
		MakePrizeTime=PrizeTime+(rand()%PrizeRandomTime)*2-
		              PrizeRandomTime;
		MakeVortexTime=VortexTime+(rand()%VortexRandomTime)*2-
		               VortexRandomTime;
		PrizeCounter=0;
		VortexCounter=0;
		InitObjectData();
		NewRockTime=SaveRockTime;
		for (i=0; i<StartRocks; i++) {
		  CreateRandomRock();
		}
		for (i=0; i<StartVortex; i++) {
		  CreateVortex();
		}
		RockTime=0;
	      }
	    }
	  }
	}
      }
      // Age all the objects
      UpdateTimes();

      if (update) {
	for (i=0; i<MaxPlayers; i++) {
	  if (ServerShips[i]) {
	    if (ServerShips[i]->Lives && !ServerShips[i]->DeadTime) {
	      ServerShips[i]->UpdatePosition();
	    }
	  }
	}
	for (i=0; i<MAX_OBJECTS; i++) {
	  if (ServerObjects[i]) {
	    ServerObjects[i]->UpdatePosition();
	  }
	}
	CheckForCollisions();
      }

      // Check for updates done once per second
      if (secondupdate && ServerGameType==SINGLE_PLAYER) {
	NewRockTime-=RockTimeSpeedup;
	if (NewRockTime < MinRockTime) {
	  NewRockTime=MinRockTime;
	}
      }

      if (EndDeathMatch) {
	EndDeathMatch=FALSE;

	pthread_mutex_unlock(ServerLock);
	sleep(1);
	pthread_mutex_lock(ServerLock);

	for (i=0; i<MaxPlayers; i++) {
	  if (ServerShips[i]) {
	    ServerShips[i]->Lives=0;
	    ServerShips[i]->DeadTime=1;
	  }
	}
	InitObjectData();

	pthread_mutex_unlock(ServerLock);
	sleep(15);
	pthread_mutex_lock(ServerLock);

	for (i=0; i<StartRocks; i++) {
	  CreateRandomRock();
	}

	for (i=0; i<MaxPlayers; i++) {
	  if (ServerShips[i]) {
	    InitPlayerData(i);
	  }
	}
      }
    }
  }
  pthread_mutex_unlock(ServerLock);

  pthread_exit(NULL);
}


void DeletePlayer(int playernum)
{
  int i;

  delete ServerShips[playernum];
  ServerShips[playernum]=NULL;
  Connections[playernum].Connected=0;
  NumConnectedPlayers--;

  for (i=0; i<MAX_OBJECTS; i++) {
    if (ServerObjects[i]) {
      if (ServerObjects[i]->type==SHOT_TYPE &&
	  ServerObjects[i]->PlayerNum==playernum) {
	delete ServerObjects[i];
	ServerObjects[i]=NULL;
      }
    }
  }
}


void InitPlayerData(int playernum)
{
  int i;

  ServerShips[playernum]=new ServerShip(((ServerWorldWidth/MaxPlayers)*
					 playernum+
					 (ServerWorldWidth/(2*MaxPlayers))),
					ServerWorldHeight/2, 0, 0,
					StartShield, 2000, StartLives);

  for (i=0; i<MAX_SOUNDS; i++) {
    Sounds[playernum][i].type=0;
  }
}


void InitObjectData()
{
  int i;

  for (i=0; i<MAX_OBJECTS; i++) {
    if (ServerObjects[i]) {
      delete ServerObjects[i];
      ServerObjects[i]=NULL;
    }
  }
}


void HandleActions(int playernum, int update)
{
  if (update) {
    if (ServerShips[playernum]->Action.left) {
      ServerShips[playernum]->Phase-=1;
    }
    if (ServerShips[playernum]->Action.right) {
      ServerShips[playernum]->Phase+=1;
    }
    if (ServerShips[playernum]->Phase >= NUM_SHIP_PHASES) {
      ServerShips[playernum]->Phase-=NUM_SHIP_PHASES;
    }
    if (ServerShips[playernum]->Phase < 0) {
      ServerShips[playernum]->Phase+=NUM_SHIP_PHASES;
    }
    if (ServerShips[playernum]->Action.thrusting) {
      ServerShips[playernum]->Accelerate();
    }
    if (ServerShips[playernum]->Action.shooting) {
      FireShot(playernum);
    }
  }

  if (ServerShips[playernum]->Action.shield &&
      ServerShips[playernum]->ShieldValue) {
    ServerShips[playernum]->ShieldOn=1;
    if (!ServerShips[playernum]->ShieldTime) {
      ServerShips[playernum]->ShieldValue-=GameSleep;
      if (ServerShips[playernum]->ShieldValue < 0) {
	ServerShips[playernum]->ShieldValue=0;
      }
    }
  } else {
    ServerShips[playernum]->ShieldOn=0;
  }
}


int CreateObject(char type, double xpos, double ypos,
		 double xvel, double yvel, int AliveTime)
{
  int i, time;

  if (AliveTime) {
    time=AliveTime;
  } else {
    time=1;
  }

  if (type==PRIZE_TYPE) {
    AddSoundAll(xpos, ypos, PrizeAppears);
  }

  for (i=0; i<MAX_OBJECTS; i++) {
    if (!ServerObjects[i]) {
      ServerObjects[i]=new ServerObject(xpos, ypos, xvel, yvel, type, time);
      ServerObjects[i]->FixPos();
      return(i);
    }
  }
  return(-1);
}


void FireShot(int playernum)
{
  int i;
  int startphase, endphase;
  double xvel, yvel;
  int time, objnum;

  AddSound(playernum, ServerShips[playernum]->x,
	   ServerShips[playernum]->y, ShotSound);

  if (ServerShips[playernum]->TripleShot) {
    startphase=-1;
    endphase=1;
  } else {
    startphase=0;
    endphase=0;
  }

  time=ShotKeepAlive;
  if (ServerShips[playernum]->LongFire) {
    time=(time*3)/2;
  }

  for (i=startphase; i<=endphase; i++) {
    if (ServerShips[playernum]->numShots >= MAX_SHOTS) {
      break;
    }
    ServerShips[playernum]->numShots++;
    CalcShotVelocity(playernum, i, &xvel, &yvel);
    objnum=CreateObject(SHOT_TYPE, ServerShips[playernum]->x+(SHIP_SIZE/2),
			ServerShips[playernum]->y+(SHIP_SIZE/2),
			xvel, yvel, time);
    ServerObjects[objnum]->PlayerNum=playernum;
  }

  if (!ServerShips[playernum]->AutoFire) {
    ServerShips[playernum]->Action.shooting=FALSE;
  }
}


void CalcShotVelocity(int playernum, int phaseoffset,
		      double *xvel, double *yvel)
{
  double angle;
  double offset;

  *xvel=ServerShips[playernum]->xvel;
  *yvel=ServerShips[playernum]->yvel;

  angle=ServerShips[playernum]->Phase+phaseoffset;
  angle=angle/NUM_SHIP_PHASES*2*M_PI;

  offset=M_PI/2.0;

  *xvel+=ShotSpeed*cos(angle+3.0*offset);
  *yvel+=ShotSpeed*sin(angle+offset);
}


void CheckForCollisions()
{
  int i, j, Lucky;

  // Check if an object collided with a ship
  for (i=0; i<MAX_OBJECTS; i++) {
    if (ServerObjects[i]) {
      for (j=0; j<MaxPlayers; j++) {
	if (ServerShips[j]) {
	  if (ServerShips[j]->Lives && !ServerShips[j]->DeadTime) {
	    if (CheckForCollision(ServerShips[j]->x, ServerShips[j]->y,
				  ServerObjects[i]->x, ServerObjects[i]->y,
				  ObjSizes[ServerShips[j]->type],
				  ObjSizes[ServerObjects[i]->type])) {
	      if (ServerObjects[i]->type==SHOT_TYPE &&
		  ServerObjects[i]->PlayerNum!=j) {
		if (ServerGameType==FREE_FOR_ALL ||
		    ServerGameType==DEATHMATCH) {
		  if (!ServerShips[j]->ShieldOn &&
		      !ServerShips[j]->ShieldTime) {
		    Lucky=ShipHit(j, i);
		    if (!Lucky) {
		      if (ServerGameType==DEATHMATCH) {
			ServerShips[ServerObjects[i]->PlayerNum]->Score+=1;
		      } else {
			ServerShips[ServerObjects[i]->PlayerNum]->Score+=ShipScore;
		      }
		      ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
		      delete ServerObjects[i];
		      ServerObjects[i]=NULL;
		      break;
		    } else {
		      AddSound(j, ServerShips[j]->x,
			       ServerShips[j]->y, LuckySound);
		    }
		    ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
		    delete ServerObjects[i];
		    ServerObjects[i]=NULL;
		    break;
		  } else {
		    BounceObjects(ServerShips[j], ServerObjects[i]);
		  }
		}
	      } else if (ServerObjects[i]->type==BIGROCK_TYPE ||
			 ServerObjects[i]->type==MEDIUMROCK_TYPE ||
			 ServerObjects[i]->type==SMALLROCK_TYPE) {
		if (!ServerShips[j]->ShieldOn && !ServerShips[j]->ShieldTime) {
		  Lucky=ShipHit(j, i);
		  if (!Lucky) {
		    if (ServerGameType==DEATHMATCH) {
		      ServerShips[j]->Score-=1;
		    } else {
		      ServerShips[j]->Score-=ShipDeadScore;
		    }
		  } else {
		    AddSound(j, ServerShips[j]->x,
			     ServerShips[j]->y, LuckySound);
		  }
		}
		RockHit(i, j);
		break;
	      } else if (ServerObjects[i]->type==STEELROID_TYPE) {
		if (!ServerShips[j]->ShieldOn && !ServerShips[j]->ShieldTime) {
		  ShipHit(j, i);
		  if (ServerGameType==DEATHMATCH) {
		    ServerShips[j]->Score-=1;
		  } else {
		    ServerShips[j]->Score-=ShipDeadScore;
		  }
		}
		if (BounceObjects(ServerObjects[i], ServerShips[j])) {
		  if (!SteelRoidHit(i, j)) {
		    break;
		  }
		}
	      } else if (ServerObjects[i]->type==PRIZE_TYPE) {
		GetPrize(i, j);
		break;
	      } else if (ServerObjects[i]->type==VORTEX_TYPE) {
		if (!ServerShips[j]->ShieldOn && !ServerShips[j]->ShieldTime) {
		  ShipHit(j, i);
		  if (ServerGameType==DEATHMATCH) {
		    ServerShips[j]->Score-=1;
		  } else {
		    ServerShips[j]->Score-=ShipDeadScore;
		  }
		} else {
		  BounceObjects(ServerShips[j], ServerObjects[i]);
		}
	      }
	    }
	  }
	}
      }
    }
  }

  // Check if two ships collided
  for (i=0; i<MaxPlayers-1; i++) {
    if (ServerShips[i]) {
      if (ServerShips[i]->Lives && !ServerShips[i]->DeadTime) {
	for (j=i+1; j<MaxPlayers; j++) {
	  if (ServerShips[j]) {
	    if (ServerShips[j]->Lives && !ServerShips[j]->DeadTime) {
	      if (CheckForCollision(ServerShips[i]->x, ServerShips[i]->y,
				    ServerShips[j]->x, ServerShips[j]->y,
				    ObjSizes[ServerShips[i]->type],
				    ObjSizes[ServerShips[j]->type])) {
		if ((ServerShips[i]->ShieldOn || ServerShips[i]->ShieldTime) &&
		    (ServerShips[j]->ShieldOn || ServerShips[j]->ShieldTime)) {
		  BounceObjects(ServerShips[i], ServerShips[j]);
		} else if (!(ServerShips[i]->ShieldOn ||
			     ServerShips[i]->ShieldTime) &&
			   (ServerShips[j]->ShieldOn ||
			    ServerShips[j]->ShieldTime)) {
		  ShipHit(i, -1);
		  if (ServerGameType==DEATHMATCH) {
		    ServerShips[j]->Score+=1;
		  } else {
		    ServerShips[j]->Score+=ShipScore;
		  }
		} else if ((ServerShips[i]->ShieldOn ||
			    ServerShips[i]->ShieldTime) &&
			   !(ServerShips[j]->ShieldOn ||
			     ServerShips[j]->ShieldTime)) {
		  ShipHit(j, -1);
		  if (ServerGameType==DEATHMATCH) {
		    ServerShips[i]->Score+=1;
		  } else {
		    ServerShips[i]->Score+=ShipScore;
		  }
		} else {
		  ShipHit(i, -1);
		  ShipHit(j, -1);
		}
	      }
	    }
	  }
	}
      }
    }
  }

  // Check if an object collided with another object
  for (i=0; i<MAX_OBJECTS-1; i++) {
    if (ServerObjects[i]) {
      for (j=i+1; j<MAX_OBJECTS; j++) {
	if (ServerObjects[j]) {
	  if (CheckForCollision(ServerObjects[i]->x, ServerObjects[i]->y,
				ServerObjects[j]->x, ServerObjects[j]->y,
				ObjSizes[ServerObjects[i]->type],
				ObjSizes[ServerObjects[j]->type])) {
	    if (ServerObjects[i]->type==SHOT_TYPE) {
	      if (ServerObjects[j]->type==SMALLROCK_TYPE ||
		  ServerObjects[j]->type==MEDIUMROCK_TYPE ||
		  ServerObjects[j]->type==BIGROCK_TYPE) {
		RockHit(j, ServerObjects[i]->PlayerNum);
		ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
		delete ServerObjects[i];
		ServerObjects[i]=NULL;
		break;
	      } else if (ServerObjects[j]->type==PRIZE_TYPE) {
		ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
		PrizeHit(j, ServerObjects[i]->PlayerNum);
		delete ServerObjects[i];
		ServerObjects[i]=NULL;
		break;
	      } else if (ServerObjects[j]->type==STEELROID_TYPE) {
		if (BounceObjects(ServerObjects[i], ServerObjects[j])) {
		  if (!SteelRoidHit(j, ServerObjects[i]->PlayerNum)) {
		    ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
		    delete ServerObjects[i];
		    ServerObjects[i]=NULL;
		    break;
		  }
		}
	      } else if (ServerObjects[j]->type==VORTEX_TYPE) {
		ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
		delete ServerObjects[i];
		ServerObjects[i]=NULL;
		break;
	      }
	    } else if (ServerObjects[i]->type==SMALLROCK_TYPE ||
		       ServerObjects[i]->type==MEDIUMROCK_TYPE ||
		       ServerObjects[i]->type==BIGROCK_TYPE) {
	      if (ServerObjects[j]->type==SHOT_TYPE) {
		ServerShips[ServerObjects[j]->PlayerNum]->numShots--;
		RockHit(i, ServerObjects[j]->PlayerNum);
		delete ServerObjects[j];
		ServerObjects[j]=NULL;
		break;
	      } else if (ServerObjects[j]->type==VORTEX_TYPE) {
		RockHit(i, -1);
		break;
	      }
	    } else if (ServerObjects[i]->type==STEELROID_TYPE) {
	      if (ServerObjects[j]->type==SHOT_TYPE) {
		if (BounceObjects(ServerObjects[i], ServerObjects[j])) {
		  if (!SteelRoidHit(i, ServerObjects[j]->PlayerNum)) {
		    ServerShips[ServerObjects[j]->PlayerNum]->numShots--;
		    delete ServerObjects[j];
		    ServerObjects[j]=NULL;
		    break;
		  }
		}
	      } else if (ServerObjects[j]->type==VORTEX_TYPE) {
		SteelRoidHit(i, -1);
		break;
	      }
	    } else if (ServerObjects[i]->type==PRIZE_TYPE) {
	      if (ServerObjects[j]->type==SHOT_TYPE) {
		ServerShips[ServerObjects[j]->PlayerNum]->numShots--;
		PrizeHit(i, ServerObjects[j]->PlayerNum);
		delete ServerObjects[j];
		ServerObjects[j]=NULL;
		break;
	      } else if (ServerObjects[j]->type==VORTEX_TYPE) {
		PrizeHit(i, -1);
		break;
	      }
	    } else if (ServerObjects[i]->type==VORTEX_TYPE) {
	      if (ServerObjects[j]->type==SMALLROCK_TYPE ||
		  ServerObjects[j]->type==MEDIUMROCK_TYPE ||
		  ServerObjects[j]->type==BIGROCK_TYPE) {
		RockHit(j, -1);
	      } else if (ServerObjects[j]->type==STEELROID_TYPE) {
		SteelRoidHit(j, -1);
	      } else if (ServerObjects[j]->type==PRIZE_TYPE) {
		PrizeHit(j, -1);
	      } else if (ServerObjects[j]->type==SHOT_TYPE) {
		ServerShips[ServerObjects[j]->PlayerNum]->numShots--;
		delete ServerObjects[j];
		ServerObjects[j]=NULL;
	      }
	    }
	  }
	}
      }
    }
  }

  for (i=0; i<MAX_PLAYERS; i++) {
    if (ServerShips[i]) {
      if (ServerShips[i]->Score>=MaxKills && ServerGameType==DEATHMATCH) {
	EndDeathMatch=TRUE;
      }
    }
  }
}


// Check if the two squares overlap
int CheckForCollision(double x1, double y1, double x2, double y2,
		      long size1, long size2)
{
  double dist;

  // Adjust positions for wraparound
  if (x2 > x1) {
    dist=x2-x1;
    if (dist > ServerWorldWidth-dist) {
      x1+=ServerWorldWidth;
    }
  } else {
    dist=x1-x2;
    if (dist > ServerWorldWidth-dist) {
      x2+=ServerWorldWidth;
    }
  }
  if (y2 > y1) {
    dist=y2-y1;
    if (dist > ServerWorldHeight-dist) {
      y1+=ServerWorldHeight;
    }
  } else {
    dist=y1-y2;
    if (dist > ServerWorldHeight-dist) {
      y2+=ServerWorldHeight;
    }
  }

  if (x2 > x1) {
    if (y2 > y1) {
      if (((x2-x1) < size1) && ((y2-y1) < size1)) {
	return(1);
      }
    } else {
      if (((x2-x1) < size1) && ((y1-y2) < size2)) {
	return(1);
      }
    }
  } else {
    if (y2 > y1) {
      if (((x1-x2) < size2) && ((y2-y1) < size1)) {
	return(1);
      }
    } else {
      if (((x1-x2) < size2) && ((y1-y2) < size2)) {
	return(1);
      }
    }
  }
  return(0);
}


int ShipHit(int playernum, int objnum)
{
  int i;

  if (objnum!=-1) {
    if (ServerShips[playernum]->LuckOfTheIrish && !(rand()%LUCK_ODDS) &&
	ServerObjects[objnum]->type!=VORTEX_TYPE &&
	ServerObjects[objnum]->type!=STEELROID_TYPE) {
      if (ServerObjects[objnum]->type==SHOT_TYPE) {
	AddSoundAll(ServerObjects[objnum]->x,
		    ServerObjects[objnum]->y, ExplosionSound);
	CreateObject(EXPLOSION_TYPE, ServerObjects[objnum]->x,
		     ServerObjects[objnum]->y, 0.0, 0.0, 150);
      }
      return(1);
    }
  }

  AddSoundAll(ServerShips[playernum]->x,
	      ServerShips[playernum]->y, ShipHitSound);
  CreateObject(SHRAPNEL1_TYPE, ServerShips[playernum]->x,
	       ServerShips[playernum]->y,
	       ServerShips[playernum]->xvel+(24.0+rand()%15)/16.0,
	       ServerShips[playernum]->yvel-(13+rand()%27)/16.0, 750);
  CreateObject(SHRAPNEL2_TYPE, ServerShips[playernum]->x,
	       ServerShips[playernum]->y,
	       ServerShips[playernum]->xvel-(36.0+rand()%15)/16.0,
	       ServerShips[playernum]->yvel-(13+rand()%27)/16.0, 750);
  CreateObject(SHIPEXPLOSION_TYPE, ServerShips[playernum]->x,
	       ServerShips[playernum]->y, 0.0, 0.0, 750);
  ServerShips[playernum]->DeadTime=ShipDeadTime;
  ServerShips[playernum]->ShieldTime=2000;
  ServerShips[playernum]->ShieldValue=StartShield;
  ServerShips[playernum]->xvel=0;
  ServerShips[playernum]->yvel=0;
  ServerShips[playernum]->Phase=0;
  ServerShips[playernum]->TripleShot=0;
  ServerShips[playernum]->LuckOfTheIrish=0;
  ServerShips[playernum]->AutoFire=0;
  ServerShips[playernum]->LongFire=0;
  ServerShips[playernum]->Speedup=0;
  ServerShips[playernum]->Action.thrusting=0;
  ServerShips[playernum]->Action.left=0;
  ServerShips[playernum]->Action.right=0;
  ServerShips[playernum]->Action.shooting=0;

  return(0);
}


void UpdateTimes()
{
  int i;

  for (i=0; i<MaxPlayers; i++) {
    if (ServerShips[i]) {
      if (ServerShips[i]->Lives) {
	if (ServerShips[i]->DeadTime) {
	  ServerShips[i]->DeadTime-=GameSleep;
	  if (ServerShips[i]->DeadTime <= 0) {
	    ServerShips[i]->DeadTime=0;
	    ServerShips[i]->x=(ServerWorldWidth/MaxPlayers)*i+
	                (ServerWorldWidth/(2*MaxPlayers));
	    ServerShips[i]->y=ServerWorldHeight/2;
	    if (ServerGameType==SINGLE_PLAYER) {
	      ServerShips[i]->Lives--;
	    }
	  }
	} else {
	  if (ServerShips[i]->ShieldTime) {
	    ServerShips[i]->ShieldTime-=GameSleep;
	    if (ServerShips[i]->ShieldTime < 0) {
	      ServerShips[i]->ShieldTime=0;
	    }
	  }
	}
      }
    }
  }

  for (i=0; i<MAX_OBJECTS; i++) {
    if (ServerObjects[i]) {
      if ((ServerObjects[i]->type == SHOT_TYPE ||
	   (ServerObjects[i]->type == PRIZE_TYPE && PrizeAliveTime) ||
	   (ServerObjects[i]->type == VORTEX_TYPE && VortexAliveTime) ||
	   ServerObjects[i]->type == SHRAPNEL1_TYPE ||
	   ServerObjects[i]->type == SHRAPNEL2_TYPE ||
	   ServerObjects[i]->type == SHIPEXPLOSION_TYPE ||
	   ServerObjects[i]->type == EXPLOSION_TYPE) &&
	  ServerObjects[i]->AliveTime) {
	ServerObjects[i]->AliveTime-=GameSleep;
	if (ServerObjects[i]->AliveTime <= 0) {
	  if (ServerObjects[i]->type == SHOT_TYPE) {
	    ServerShips[ServerObjects[i]->PlayerNum]->numShots--;
	  }
	  delete ServerObjects[i];
	  ServerObjects[i]=NULL;
	}
      }
    }
  }
}


void CreateRandomRock()
{
  int objnum;
  double x, y;
  int type;

  if (SteelRoidOdds) {
    if (rand()%SteelRoidOdds) {
      type=BIGROCK_TYPE;
    } else {
      type=STEELROID_TYPE;
    }
  } else {
    type=BIGROCK_TYPE;
  }

  objnum=CreateObject(type, (((rand()%256)/256.0)*ServerWorldWidth),
		      (((rand()%256)/256.0)*ServerWorldHeight),
		      ((rand()%60)-30)/16.0, ((rand()%60)-30)/16.0, 1);
  while (ServerObjects[objnum]->CheckIfTooClose()) {
    ServerObjects[objnum]->x=(((rand()%256)/256.0)*ServerWorldWidth);
    ServerObjects[objnum]->y=(((rand()%256)/256.0)*ServerWorldHeight);
  }
  if (type==STEELROID_TYPE) {
    ServerObjects[objnum]->HitPoints=SteelRoidHitPoints;
  }
}


void CreateVortex()
{
  int objnum;
  double x, y;
  int time;

  if (VortexAliveTime) {
    time=VortexAliveTime;
  } else {
    time=1;
  }

  objnum=CreateObject(VORTEX_TYPE, (((rand()%256)/256.0)*ServerWorldWidth),
		      (((rand()%256)/256.0)*ServerWorldHeight), 0, 0, time);
  if (objnum!=-1) {
    ServerObjects[objnum]->GravityStrength=GravityStrength;
    while (ServerObjects[objnum]->CheckIfTooClose()) {
      ServerObjects[objnum]->x=(((rand()%256)/256.0)*ServerWorldWidth);
      ServerObjects[objnum]->y=(((rand()%256)/256.0)*ServerWorldHeight);
    }
  }
}


void RockHit(int objnum, int playernum)
{
  int i, randnum;

  AddSoundAll(ServerObjects[objnum]->x,
	      ServerObjects[objnum]->y, ExplosionSound);
  CreateObject(EXPLOSION_TYPE, ServerObjects[objnum]->x,
	       ServerObjects[objnum]->y, 0.0, 0.0, 150);

  if (ServerObjects[objnum]->type==BIGROCK_TYPE) {
    if ((ServerGameType==FREE_FOR_ALL || ServerGameType==SINGLE_PLAYER) &&
	playernum!=-1) {
      ServerShips[playernum]->Score+=BigRockScore;
    }
    randnum=rand()%3+1;
    for (i=0; i<randnum; i++) {
      CreateObject(MEDIUMROCK_TYPE,
		   ServerObjects[objnum]->x+ObjSizes[BIGROCK_TYPE]/2-
		   ObjSizes[MEDIUMROCK_TYPE]/2,
		   ServerObjects[objnum]->y+ObjSizes[BIGROCK_TYPE]/2-
		   ObjSizes[MEDIUMROCK_TYPE]/2,
		   ServerObjects[objnum]->xvel+(rand()%60-30)/16.0,
		   ServerObjects[objnum]->yvel+(rand()%60-30)/16.0, 0);
    }
  } else if (ServerObjects[objnum]->type==MEDIUMROCK_TYPE) {
    if ((ServerGameType==FREE_FOR_ALL || ServerGameType==SINGLE_PLAYER) &&
	playernum!=-1) {
      ServerShips[playernum]->Score+=MediumRockScore;
    }
    randnum=rand()%3+1;
    for (i=0; i<randnum; i++) {
      CreateObject(SMALLROCK_TYPE,
		   ServerObjects[objnum]->x+ObjSizes[MEDIUMROCK_TYPE]/2-
		   ObjSizes[SMALLROCK_TYPE]/2,
		   ServerObjects[objnum]->y+ObjSizes[MEDIUMROCK_TYPE]/2-
		   ObjSizes[SMALLROCK_TYPE]/2,
		   ServerObjects[objnum]->xvel+(rand()%60-30)/16.0,
		   ServerObjects[objnum]->yvel+(rand()%60-30)/16.0, 0);
    }
  } else if (ServerObjects[objnum]->type==SMALLROCK_TYPE) {
    if ((ServerGameType==FREE_FOR_ALL || ServerGameType==SINGLE_PLAYER) &&
	playernum!=-1) {
      ServerShips[playernum]->Score+=SmallRockScore;
    }
  }

  delete ServerObjects[objnum];
  ServerObjects[objnum]=NULL;
}


int SteelRoidHit(int objnum, int playernum)
{
  ServerObjects[objnum]->HitPoints--;

  if (!ServerObjects[objnum]->HitPoints && playernum!=-1) {
    if (ServerGameType==FREE_FOR_ALL || ServerGameType==SINGLE_PLAYER) {
      ServerShips[playernum]->Score+=SteelRoidScore;
    }
    KillSteelRoid(objnum);
    return(0);
  } else if (playernum==-1) {
    KillSteelRoid(objnum);
    return(0);
  } else {
    AddSoundAll(ServerObjects[objnum]->x, ServerObjects[objnum]->y, SteelHit);
    return(1);
  }
}


void KillSteelRoid(int objnum)
{
  AddSoundAll(ServerObjects[objnum]->x,
	      ServerObjects[objnum]->y, ExplosionSound);
  CreateObject(EXPLOSION_TYPE, ServerObjects[objnum]->x,
	       ServerObjects[objnum]->y, 0.0, 0.0, 150);
  delete ServerObjects[objnum];
  ServerObjects[objnum]=NULL;
}


void GetPrize(int objnum, int playernum)
{
  int prizetype;

  AddSound(playernum, ServerObjects[objnum]->x,
	   ServerObjects[objnum]->y, GotPrize);

  delete ServerObjects[objnum];
  ServerObjects[objnum]=NULL;

  prizetype=rand()%NUM_PRIZE_TYPES+1;

  if (prizetype==TRIPLE_SHOT) {
    ServerShips[playernum]->TripleShot=1;
  } else if (prizetype==SHIELD_PRIZE) {
    ServerShips[playernum]->ShieldValue+=(rand()%(ServerMaxShield/4))+
                                   ServerMaxShield/4;
    if (ServerShips[playernum]->ShieldValue > ServerMaxShield) {
      ServerShips[playernum]->ShieldValue=ServerMaxShield;
    }
  } else if (prizetype==LUCK_OF_THE_IRISH) {
    ServerShips[playernum]->LuckOfTheIrish=1;
  } else if (prizetype==AUTOFIRE) {
    ServerShips[playernum]->AutoFire=1;
  } else if (prizetype==LONGFIRE) {
    ServerShips[playernum]->LongFire=1;
  } else if (prizetype==SPEEDUP) {
    ServerShips[playernum]->Speedup=1;
  }
}


void PrizeHit(int objnum, int playernum)
{
  if (playernum!=-1) {
    AddSound(playernum, ServerObjects[objnum]->x,
	     ServerObjects[objnum]->y, IdiotSound);
  }
  CreateObject(EXPLOSION_TYPE, ServerObjects[objnum]->x,
	       ServerObjects[objnum]->y, 0.0, 0.0, 150);
  delete ServerObjects[objnum];
  ServerObjects[objnum]=NULL;
}


void AddSound(int playernum, double x, double y, char type)
{
  int i;

  for (i=0; i<NUM_SOUNDS; i++) {
    if (!Sounds[playernum][i].type) {
      Sounds[playernum][i].x=(long)x;
      Sounds[playernum][i].y=(long)y;
      Sounds[playernum][i].type=type;
      break;
    }
  }
}


void AddSoundAll(double x, double y, char type)
{
  int i;

  for (i=0; i<MaxPlayers; i++) {
    AddSound(i, x, y, type);
  }
}


int BounceObjects(ServerObject *obj1, ServerObject *obj2)
{
  double AngleOfImpact;
  double VelAngle1, VelAngle2;
  double Accel1, Accel2;
  double Vel1, Vel2;
  double Dist1, Dist2;

  Dist1=CalcDistance(obj1->x, obj1->y, obj1->Size,
		     obj2->x, obj2->y, obj2->Size);
  Dist2=CalcDistance(obj1->x+obj1->xvel/100.0, obj1->y+obj1->yvel/100.0,
		     obj1->Size,
		     obj2->x+obj2->xvel/100.0, obj2->y+obj2->yvel/100.0,
		     obj2->Size);

  if (Dist1 > Dist2) {
    Vel1=sqrt(obj1->xvel*obj1->xvel+obj1->yvel*obj1->yvel);
    Vel2=sqrt(obj2->xvel*obj2->xvel+obj2->yvel*obj2->yvel);

    if (obj1->xvel!=0) {
      VelAngle1=atan(obj1->yvel/obj1->xvel);
      if (obj1->xvel<0) {
	VelAngle1+=M_PI;
      }
    } else {
      if (obj1->yvel>0) {
	VelAngle1=M_PI/2.0;
      } else {
	VelAngle1=-M_PI/2.0;
      }
    }
    if (obj2->xvel!=0) {
      VelAngle2=atan(obj2->yvel/obj2->xvel);
      if (obj2->xvel<0) {
	VelAngle2+=M_PI;
      }
    } else {
      if (obj2->yvel>0) {
	VelAngle2=M_PI/2.0;
      } else {
	VelAngle2=-M_PI/2.0;
      }
    }

    AngleOfImpact=CalcAngle(obj1->x, obj1->y, obj1->Size,
			    obj2->x, obj2->y, obj2->Size);

    if (obj1->Weight && obj2->Weight) {
      Accel1=-(2.0/(1.0+((double)obj1->Weight/(double)obj2->Weight)))*
	((Vel1*cos(VelAngle1-AngleOfImpact))-
	 (Vel2*cos(VelAngle2-AngleOfImpact)));
      Accel2=-Accel1*((double)obj1->Weight/(double)obj2->Weight);
    } else if (obj1->Weight && !obj2->Weight) {
      Accel1=-2.0*((Vel1*cos(VelAngle1-AngleOfImpact))-
		  (Vel2*cos(VelAngle2-AngleOfImpact)));
      Accel2=0;
    } else if (!obj1->Weight && obj2->Weight) {
      Accel1=0;
      Accel2=2.0*((Vel1*cos(VelAngle1-AngleOfImpact))-
		  (Vel2*cos(VelAngle2-AngleOfImpact)));
    } else {
      Accel1=0;
      Accel2=0;
    }

    obj1->xvel+=Accel1*cos(AngleOfImpact);
    obj1->yvel+=Accel1*sin(AngleOfImpact);
    obj2->xvel+=Accel2*cos(AngleOfImpact);
    obj2->yvel+=Accel2*sin(AngleOfImpact);
    return(1);

  } else {
    return(0);
  }
}


double CalcDistance(double x1, double y1, int size1,
		    double x2, double y2, int size2)
{
  double xdist, ydist;

  x1=x1+size1/2.0;
  y1=y1+size1/2.0;
  x2=x2+size2/2.0;
  y2=y2+size2/2.0;

  if (x1 > x2) {
    xdist=x1-x2;
  } else {
    xdist=x2-x1;
  }
  if (xdist > ServerWorldWidth-xdist) {
    xdist=ServerWorldWidth-xdist;
  }
  if (y1 > y2) {
    ydist=y1-y2;
  } else {
    ydist=y2-y1;
  }
  if (ydist > ServerWorldHeight-ydist) {
    ydist=ServerWorldHeight-ydist;
  }

  return(sqrt(xdist*xdist+ydist*ydist));
}


// Calculates the angle between two points
double CalcAngle(double x1, double y1, int size1,
		 double x2, double y2, int size2)
{
  double value;

  x1=x1+size1/2.0;
  y1=y1+size1/2.0;
  x2=x2+size2/2.0;
  y2=y2+size2/2.0;  

  if (x1 > x2) {
    if ((x1-x2) > ServerWorldWidth/2) {
      x2+=ServerWorldWidth;
    }
  } else {
    if ((x2-x1) > ServerWorldWidth/2) {
      x1+=ServerWorldWidth;
    }
  }
  if (y1 > y2) {
    if ((y1-y2) > ServerWorldHeight/2) {
      y2+=ServerWorldHeight;
    }
  } else {
    if ((y2-y1) > ServerWorldHeight/2) {
      y1+=ServerWorldHeight;
    }
  }
  if (x2!=x1) {
    value=(y2-y1)/(x2-x1);
    value=atan(value);
    if (x1 > x2) {
      value+=M_PI;
    }
  } else {
    if (y2-y1>0) {
      value=M_PI/2.0;
    } else {
      value=-M_PI/2.0;
    }
  }

  return(value);
}
