#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>


#include "defines.h"
#include "servergame.h"
#include "serverdata.h"
#include "servership.h"
#include "objdata.h"
#include "mainplayerthread.h"
#include "scorevals.h"
#include "sounddefs.h"


ServerObject::ServerObject(double xpos, double ypos, double xvelocity,
			   double yvelocity, char objtype, int time)
{
  x=xpos;
  y=ypos;
  xvel=xvelocity;
  yvel=yvelocity;
  type=objtype;
  AliveTime=time;
  Size=ObjSizes[type];
  Weight=ObjWeights[type];
}


void ServerObject::UpdatePosition()
{
  x+=xvel;
  y+=yvel;
  FixPos();

  // Check if object affected by gravity
  if (type == BIGROCK_TYPE ||
      type == MEDIUMROCK_TYPE ||
      type == SMALLROCK_TYPE ||
      type == STEELROID_TYPE ||
      type == PRIZE_TYPE ||
      type == SHIP_TYPE ||
      type == SHOT_TYPE) {
    CalcGravity();
  }
}


void ServerObject::CalcGravity()
{
  int i;
  double dist, force, angle;

  for (i=0; i<MAX_OBJECTS; i++) {
    if (ServerObjects[i]) {
      if (ServerObjects[i]->type==VORTEX_TYPE &&
	  (ServerObjects[i]->x!=x || ServerObjects[i]->y!=y)) {
	
	dist=CalcDistance(x, y, Size, ServerObjects[i]->x,
			  ServerObjects[i]->y, ServerObjects[i]->Size);
	if (dist < 1) {
	  dist=1;
	}
	force=ServerObjects[i]->GravityStrength/(dist*dist);
	angle=CalcAngle(x, y, Size, ServerObjects[i]->x,
			ServerObjects[i]->y, ServerObjects[i]->Size);
	UpdateSpeed(cos(angle)*force, sin(angle)*force);
      }
    }
  }
}


void ServerObject::UpdateSpeed(double xaccel, double yaccel)
{
  double velocity;

  xvel+=xaccel;
  yvel+=yaccel;

  velocity=sqrt(xvel*xvel+yvel*yvel);

  if (type != SHOT_TYPE) {
    if (velocity > MaxSpeed) {
      xvel*=(MaxSpeed/velocity);
      yvel*=(MaxSpeed/velocity);
    }
  } else {
    if (velocity > MaxSpeed + ShotSpeed) {
      xvel*=(MaxSpeed/velocity);
      yvel*=(MaxSpeed/velocity);
    }
  }
}


void ServerObject::FixPos()
{
  if (x < 0) {
    x=x+ServerWorldWidth;
  }
  if (x >= ServerWorldWidth) {
    x=x-ServerWorldWidth;
  }
  if (y < 0) {
    y=y+ServerWorldHeight;
  }
  if (y >= ServerWorldHeight) {
    y=y-ServerWorldHeight;
  }
}


// Check if object is created too close to a ship
int ServerObject::CheckIfTooClose()
{
  int i;
  int xdist, ydist;

  for (i=0; i<MaxPlayers; i++) {
    if (ServerShips[i]) {
      if (ServerShips[i]->Lives) {
	xdist=abs((int)(ServerShips[i]->x-x));
	ydist=abs((int)(ServerShips[i]->y-y));

	// Check for wrap around
	if (xdist > ServerWorldWidth-xdist) {
	  xdist=abs(xdist-ServerWorldWidth);
	}
	if (ydist > ServerWorldHeight-ydist) {
	  ydist=abs(ydist-ServerWorldHeight);
	}

	if (xdist<SAFE_DISTANCE &&
	    ydist<SAFE_DISTANCE) {
	  return(1);
	}
      }
    }
  }
  return(0);
}
