#ifndef NOSOUND
#include <AL/al.h>
#include <AL/alc.h>
#include <AL/alut.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>


#include "globals.h"


#ifndef NOSOUND
typedef struct {
  ALsizei size;
  ALsizei bits;
  ALsizei freq;
  ALsizei format;
  void *wave;
} wavSound;


void playwave(int soundnum);
void LoadWavs();
ALboolean SourceIsPlaying(ALuint sid);


ALuint sounddesc[MAX_SOUNDS];
ALuint soundbuf[MAX_SOUNDS];
wavSound Wavs[NUM_SOUNDS];


void initsound()
{
  int i;

  for (i=0; i<MAX_SOUNDS; i++) {
    sounddesc[i]=0;
    soundbuf[i]=0;
  }
  LoadWavs();
}
#endif


// Function to play a sound file
void playsound(int soundnum)
{
#ifndef NOSOUND
  playwave(soundnum);
#endif
}


#ifndef NOSOUND
void playwave(int soundnum)
{
  ALsizei size;
  ALsizei bits;
  ALsizei freq;
  ALsizei format;
  ALboolean err;
  void *wave;
  int i;

  // Find an open sound buffer to add sound to, and clean up existing buffers
  for (i=0; i<MAX_SOUNDS; i++) {
    if(SourceIsPlaying(sounddesc[i]) == AL_TRUE) {
      if (i==MAX_SOUNDS) {
	return;
      }
      continue;
    }
    if (sounddesc) {
      alDeleteBuffers(1, &soundbuf[i]);
      alDeleteSources(1, &sounddesc[i]);
      soundbuf[i]=0;
      sounddesc[i]=0;
    }
    alGenBuffers(1, &soundbuf[i]);
    break;
  }

  alBufferData(soundbuf[i], Wavs[soundnum].format,
	       Wavs[soundnum].wave, Wavs[soundnum].size,
	       Wavs[soundnum].freq);

  alGenSources(1, &sounddesc[i]);
  alSourcei(sounddesc[i], AL_BUFFER, soundbuf[i]);
  alSourcei(sounddesc[i], AL_LOOPING, AL_FALSE);

  alSourcePlay(sounddesc[i]);
}


// Function to check if a sound is playing in the buffer sid
ALboolean SourceIsPlaying(ALuint sid)
{
  ALint state;

  if (alIsSource(sid) == AL_FALSE) {
    return(AL_FALSE);
  }

  state = AL_INITIAL;
  alGetSourceiv(sid, AL_SOURCE_STATE, &state);

  switch(state) {
  case AL_PLAYING:
  case AL_PAUSED:
    return(AL_TRUE);
  default:
    break;
  }

  return(AL_FALSE);
}


void LoadWavs()
{
  int i;
  char filename[80];

  for (i=0; i<NUM_SOUNDS; i++) {
#ifdef WINDOWS
    sprintf(filename, "%s/wavs/%d.wav", DATADIR, i);
#else
    sprintf(filename, "%s/atr/wavs/%d.wav", DATADIR, i);
#endif
    alutLoadWAV(filename, &Wavs[i].wave, &Wavs[i].format,
		&Wavs[i].size, &Wavs[i].bits, &Wavs[i].freq);
  }
}
#endif
